#!/usr/bin/perl
#
# perl script to link into the current working directory those files
# need for updating the geodyn bih tables.  The perl script will
# call the fortran program, bih_master_update, to update the
# master bih file with any previous, current, estimated and 
# predicted data used in the table.  Next, the geodyn program
# to create the binary geodyn bih table will be executed.
#
# Programmer:  Terry Williams                Date:  Sept 28, 2001
#
#  Modified:  May 6, 2011
#    Version number designation no longer supported by NOAA.  Only yearly file is
#    updated


# initialize environment and global variables
use Cwd;
use IPC::Open2;

$bindir = "$ENV{'BINDIR'}";
chomp($bindir);
do ("${bindir}/init.pl");

# check mode
if( @ARGV ) {
 if( $ARGV[0] =~ /^-h$/ ) {
   usage();
   exit();
 }
}

# Open the import defaults file
open (SYSM,"${sysfiles}/import_defaults")  ||
   die "can't open ${sysfiles}/import_defaults : $!\n";
flock(SYSM,$LOCK_EX);

# Find import variables needed for gps data import
while (<SYSM>) {
  @values = split(/\s+/, $_);
  if(/^tables_localdir/) {
    $local_dir = $values[2];
    $fq_local_dir = sprintf("%s%s",${datadir},$local_dir);
    last;
  }
}

flock(SYSM,$LOCK_UN);
close(SYSM);

# initialize table program name
   $bih_exec_iono = "tables0110_iono";
   $bih_exec = "tables0110";

# cd into a temporary work dir
do("${bindir}/mktemp.pl");

# link in file need for updating the bih table

# utc files
  if( -e "${fq_local_dir}/utc/UTC-TAI.history" ) {
     #symlink( "${fq_local_dir}/utc/UTC-TAI.history", "UTC-TAI.history" ); 
     `cp ${fq_local_dir}/utc/UTC-TAI.history .`;
     `cp ${fq_local_dir}/utc/bulc_import_log .`;

     $date_modified = sprintf("%s/file_date_modified.pl UTC-TAI.history",${bindir} );
     $pid = open2('OUTPUT','INPUT',$date_modified);
     $file_date = scalar(<OUTPUT>);
     wait;
     close (OUTPUT);
     close (INPUT);
     chomp($file_date);

     open(LOG, ">>bulc_import_log") ||
       die "can't open bulc_import_log : $!\n";
     seek(LOG,0,2);
     print LOG "\n\n  Date UTC_TAI.history last modified: $file_date\n";
     close( LOG );
     
  }

# eop files
  #  eopc04.62-now has been replaced with eopc04_IAU2000.62-now
  if( -e "${fq_local_dir}/eop/eopc04_IAU2000.62-now" ) {
     #symlink( "${fq_local_dir}/eop/eopc04_IAU2000.62-now", "eopc04_IAU2000.62-now" ); 
     `cp ${fq_local_dir}/eop/eopc04_IAU2000.62-now .`;
     `cp ${fq_local_dir}/eop/eop_import_log .`;

     $date_modified = sprintf("%s/file_date_modified.pl eopc04_IAU2000.62-now",${bindir} );
     $pid = open2('OUTPUT','INPUT',$date_modified);
     $file_date = scalar(<OUTPUT>);
     wait;
     close (OUTPUT);
     close (INPUT);
     chomp($file_date);

     open(LOG, ">>eop_import_log") ||
       die "can't open eop_import_log : $!\n";
     seek(LOG,0,2);
     print LOG "\n\n  Date eopc04_IAU2000.62-now last modified: $file_date\n";
     close( LOG );
  }
  # ser7 has been replaced with finals2000A.daily
  if( -e "${fq_local_dir}/eop/finals2000A.daily" ) {
     #symlink( "${fq_local_dir}/eop/finals2000A.daily", "finals2000A.daily" ); 
     `cp ${fq_local_dir}/eop/finals2000A.daily .`;
     `cp ${fq_local_dir}/eop/bul_import_log .`;

     $date_modified = sprintf("%s/file_date_modified.pl finals2000A.daily",${bindir} );
     $pid = open2('OUTPUT','INPUT',$date_modified);
     $file_date = scalar(<OUTPUT>);
     wait;
     close (OUTPUT);
     close (INPUT);
     chomp($file_date);

     open(LOG, ">>bul_import_log") ||
       die "can't open bul_import_log : $!\n";
     seek(LOG,0,2);
     print LOG "\n\n  Date finals2000A.daily last modified: $file_date\n";
     close( LOG );
  }

# sgo files
  # always obtain the latest version available
  # get current year and month
  $year = `date +%Y`;
  chomp($year);
  $prev_year = $year - 1;
  
  $file_out = "$year";
  $file_prev_out = "$prev_year";

  `cp ${fq_local_dir}/sgo/kpap_import_log .`;
  open(LOG, ">>kpap_import_log") ||
    die "can't open kpap_import_log : $!\n";
  seek(LOG,0,2);

  if( -e "${fq_local_dir}/sgo/$file_out" ) {
     #symlink( "${fq_local_dir}/sgo/$file_out",  "$file_out" ); 
     `cp ${fq_local_dir}/sgo/$file_out .`;

     $date_modified = sprintf("%s/file_date_modified.pl $file_out",${bindir} );
     $pid = open2('OUTPUT','INPUT',$date_modified);
     $file_date = scalar(<OUTPUT>);
     wait;
     close (OUTPUT);
     close (INPUT);
     chomp($file_date);
     print LOG "\n\n  Date ${file_out} last modified: $file_date\n";
  }
  if( -e "${fq_local_dir}/sgo/${prev_year}" ) {
     #symlink( "${fq_local_dir}/sgo/${prev_year}",  "$file_prev_out" ); 
     `cp ${fq_local_dir}/sgo/${prev_year}  $file_prev_out`;

     $date_modified = sprintf("%s/file_date_modified.pl $file_prev_out",${bindir} );
     $pid = open2('OUTPUT','INPUT',$date_modified);
     $file_date = scalar(<OUTPUT>);
     wait;
     close (OUTPUT);
     close (INPUT);
     chomp($file_date);
     print LOG "\n\n  Date ${file_prev_out} last modified: $file_date\n";
  }
  close( LOG );

  `cp ${fq_local_dir}/sgo/sgo_import_log .`;
  open(LOG, ">>sgo_import_log") ||
    die "can't open sgo_import_log : $!\n";
  seek(LOG,0,2);

  if( -e "${fq_local_dir}/sgo/DGD.txt" ) {
     #symlink( "${fq_local_dir}/sgo/DGD.txt",  "DGD.txt" ); 
     `cp ${fq_local_dir}/sgo/DGD.txt .`;

     $date_modified = sprintf("%s/file_date_modified.pl DGD.txt",${bindir} );
     $pid = open2('OUTPUT','INPUT',$date_modified);
     $file_date = scalar(<OUTPUT>);
     wait;
     close (OUTPUT);
     close (INPUT);
     chomp($file_date);
     print LOG "\n\n  Date DGD.txt last modified: $file_date\n";
  }
  if( -e "${fq_local_dir}/sgo/DSD.txt" ) {
     #symlink( "${fq_local_dir}/sgo/DSD.txt",  "DSD.txt" ); 
     `cp ${fq_local_dir}/sgo/DSD.txt .`;

     $date_modified = sprintf("%s/file_date_modified.pl DSD.txt",${bindir} );
     $pid = open2('OUTPUT','INPUT',$date_modified);
     $file_date = scalar(<OUTPUT>);
     wait;
     close (OUTPUT);
     close (INPUT);
     chomp($file_date);
     print LOG "\n\n  Date DSD.txt last modified: $file_date\n";
  }
  if( -e "${fq_local_dir}/sgo/45DF.txt" ) {
     #symlink( "${fq_local_dir}/sgo/45DF.txt", "45DF.txt" ); 
     `cp ${fq_local_dir}/sgo/45DF.txt .`;

     $date_modified = sprintf("%s/file_date_modified.pl 45DF.txt",${bindir} );
     $pid = open2('OUTPUT','INPUT',$date_modified);
     $file_date = scalar(<OUTPUT>);
     wait;
     close (OUTPUT);
     close (INPUT);
     chomp($file_date);
     print LOG "\n\n  Date 45DF.txt last modified: $file_date\n";
  }
  close( LOG );

# iono file
  if( -e "${fq_local_dir}/iono/ig_rz.dat" ) {
     symlink( "${fq_local_dir}/iono/ig_rz.dat",  "ig_rz.dat" ); 
     `cp ${fq_local_dir}/iono/iono_import_log .`;
     open(LOG, ">>iono_import_log") ||
       die "can't open iono_import_log : $!\n";
     seek(LOG,0,2);

     $date_modified = sprintf("%s/file_date_modified.pl ig_rz.dat",${bindir} );
     $pid = open2('OUTPUT','INPUT',$date_modified);
     $file_date = scalar(<OUTPUT>);
     wait;
     close (OUTPUT);
     close (INPUT);
     chomp($file_date);
     print LOG "\n\n  Date ig_rz.dat last modified: $file_date\n";
     close( LOG );
  }

# link in master bih table
#DEP
#  `cp ${geodyn}/data/master_bih.table_iono master_bih.table`;

#DEP
# update_master_bih_table();
# rename("master_bih.table", "master_bih.table_prev");

  `cp ${geodyn}/data/master_bih.table master_bih.table`;
  exec_tables($bih_exec_iono);

  # copy master bih table files to system geodyn directory
  rename( "master_bih.table_new", "master_bih.table_iono");
  rename( "fort.14", "gdntable.data_iono");
  unlink( "fort.13" );
  unlink( "fort.11" );

  # truncate master file and create tables without ionosphere values
#DEP one more
#  `cp master_bih.table_iono master_bih.table`;
  open (BIH,"master_bih.table")  ||
      die "can't open newly created master_bih.tables : $!\n";
  while( <BIH> ) {
    if(/^IONIND/) {
       last;
    }
   $lastpos = tell(BIH);
  }
   
  truncate("master_bih.table", $lastpos);
  close(BIH);

  symlink("master_bih.table", "fort.11");
  exec_tables($bih_exec);

  rename( "fort.14", "gdntable.data");
  unlink( "fort.13" );
  unlink( "fort.11" );

  # copy all tables to geodyn data directory
  # if size of error file equal 0, do copy
  if(  (stat("bih_tables_err"))[7] == 0 && (stat("master_bih_update.log"))[7] == 0 &&
       ! -e "core" ) {
     `cp master_bih.table ${geodyn}/data`;
     `cp gdntable.data ${geodyn}/data`;
  }


##########################################################################
sub exec_tables {

  my( $bih_exec ) = @_;


  #setup execution of tables program
   open (INPT,">fort.4")  ||
      die "can't open fort.4 : $!\n";
   # write today's date on title
   $date_today =`date`;
   @dates = split(/\s+/, $date_today);
   $date1 = sprintf("%2s%2s%2s",$dates[1],$dates[2],substr($dates[5],2,2));
   $date1 =~ tr/' '/0/;
   $title = sprintf("TITLE              %6s",$date1);
   print INPT "$title\n";
   close(INPT);

   open (UPDT,">fort.12")  ||
      die "can't open fort.12 : $!\n";
   close(UPDT);
  
   symlink("master_bih.table_new", "fort.11");
  
   # submit Geodyn tables program
   `${bindir}/${bih_exec} > bih_tables_out 2> bih_tables_err`;
    wait;

}



##########################################################################
sub update_master_bih_table {

  # update master bih table
  open(RPT,">master_bih_update.report") ||
     die "can't open master_bih_update.report : $!\n";
  print RPT "                        GEODYN MASTER BIH TABLE UPDATE\n\n";
  $stars = "*" x 80;
  $dashes = "-" x 80;
  print RPT "$stars\nIMPORT SUMMARY:\n";
  print RPT "\n\n The following files were acquired for updating the GEODYN Master BIH Table:\n";
  print RPT "\n$dashes\n";
  @log_files = glob("*_log");
  foreach $log_file (@log_files) {
    open(LOG,"$log_file") || die "can't open $log_file: $!\n";
    @lines = ();
    @lines = <LOG>;
    foreach $line (@lines) {
       # check for files not for geodyn 
       if($line =~ /bulb.dat/) {
          next;
       } elsif($line =~ /bulletinc.dat/) {
          next;
       } elsif($line =~ /eopc04/) {
          if( $line !~ /now/ ) {
            next;
          }
       } else {
       }
       print RPT $line;
    }
    print RPT "$dashes\n";
    close(LOG);
  }
  print RPT "$stars\n";
  close( RPT );
  
  `${bindir}/bih_master_update`;
   wait;
}
