MODULE Date_Time_Module
!--------------------------------------------------------------------------
!
!  Module to define all the various date and time change routines using forms 
!  of Modified Julian day and Julian day. Operations include:
!
!     YMD_MJD( YMD, MJD ) - conversion from YYMMDD to MJD
!     MJD_YMD( MJD, YMD ) - conversion from MJD to YYMMDD
!     DOY_MJD( IY, DOY, MJD ) - conversion from year, day of year to MJD
!     MJD_DOY( MJD, IY, DOY ) - conversion from MJD to year, day of year
!     YMDHMS_MJDDAY( YMD, HMS, MJD, FDAY ) - conversion from YYMMDDHMS
!                            to MJD, FDAY
!     MJDDAY_YMDHMS( MJD, FDAY, YMD, HMS ) - conversion from MJD, FDAY
!                            to YMD, HMS
!     MJDSEC_YMDHMS( MJDSEC, YMD, HMS ) - conversion from MJDSEC
!                            to YMD, HMS
!     MJDSEC_FSEC_YMDHMS( MJDSEC, FSEC, YMD, HMS ) - conversion from MJDSEC,
!                            FSEC to YMD, HMS
!     YMDHMS_MJDSEC( YMD, HMS, MJDSEC ) - conversion from YMD, HM
!                            SECS to MJDSEC
!     YMDHMS_MJDSEC_FSEC( YMD, HMS, MJDSEC, FSEC ) - conversion from YMD, HM
!                            SECS to MJDSEC, FSEC
!     MJDSEC_FSEC_ADD( MJDSEC_1, FSEC_1, MJDSEC_2, FSEC_2, MJDSEC_SUM, FSEC_SUM ) - 
!                            add/substract MJDSEC_2, FSEC_2 to/from
!                            MJDSEC_1, FSEC_1 and return MJDSEC_SUM, FSEC_SUM
!     YMDHMS_GPSWEEK( YMD, HMS, GPSWEEK, DAY_OF_WEEK ) - conversion from
!                            YMD, HMS to GPS week from 1/6/80 00:00:00.
!     YRDOY_GPSWEEK( YRDOY, GPSWEEK, DAY_OF_WEEK ) - conversion from year, 
!                            day of year to GPS week from 1/6/80 00:00:00.
!     GPSWEEK_DOY( GPSWEEK, IY, DOY ) - conversion from GPSWEEK to DOY with
!                            a continuous week count from 1/6/80
!     TIME_ROUND( YMD, HMS, DELTA ) - round YMD HMS by the delta value
!     MJD_YEAR( MJD_Secs, Year, MJD_Year_Secs ) - get Jan 0, year from MJD secs
!     MJM_MJD (MJM, MJD, DAY_FRAC) - convert from VMSI MJM to MJD
!     MJD_MJM ( MJD, DAY_FRAC, MJM ) -convert from MJD + DAY_FRAC to VMSI MJM 
!     SET_TIME_REF( JD_Reference ) - set Julian Day reference (default =
!                                    2430000.5)
!      
!--------------------------------------------------------------------------

 IMPLICIT NONE

 INTEGER, PARAMETER :: DB = SELECTED_REAL_KIND(14)

 INTEGER, PARAMETER :: MD1900 = 15384
 INTEGER, PARAMETER :: IY1900 = 1900
 INTEGER, PARAMETER :: IY200 = 2000
 INTEGER, PARAMETER :: YMD40 = 19400101
 INTEGER, PARAMETER :: No_Days = 366
 INTEGER, PARAMETER :: Days_Per_Year = 36525
 INTEGER, PUBLIC :: MJD_Offset = 30000
 INTEGER, DIMENSION(13,2), PARAMETER ::                                 &
           Month = reshape((/ 0,31,59,90,120,151,181,212,243,273,304,334,365,&
                    0,31,60,91,121,152,182,213,244,274,305,335,366/),(/13,2/))
 REAL(KIND = DB), PARAMETER :: HourDay = 24.0_DB
 REAL(KIND = DB), PARAMETER :: SecDay = 86400.0_DB
 REAL(KIND = DB), PARAMETER :: SecHour = 3600.0_DB
 REAL(KIND = DB), PARAMETER :: SecMin = 60.0_DB
 REAL(KIND=DB), PARAMETER :: JD_Offset = 2400000.5


 INTEGER, PARAMETER :: IMIN_DAY = 1440
 REAL(KIND = DB), PARAMETER :: XMIN_DAY = 1440.0_DB



    type  ymd_date
        integer :: yymmdd
        integer :: hhmm
        REAL(KIND = DB) :: sec
    end type  ymd_date






 CONTAINS





!-DOY_MJD------------------------------------------------------------------
!
!  Subroutine to convert from Year, Day of Year to MJD
!
!  Input:
!   IY - year
!   DOY - Day of Year
!
!  Output:
!    MJD - Modified Julian Date
!
!--------------------------------------------------------------------------

SUBROUTINE DOY_MJD( IY, DOY, MJD )

  INTEGER, INTENT(IN) :: IY, DOY
  INTEGER, INTENT(OUT) :: MJD

  INTEGER :: IYear, NLpYrs, NDays

  IYear = IY
  IF ( IY >= IY1900 ) IYear = IY - IY1900
  NLpYrs = (IYear - 1)/4
! write(*,*) "nlpyrs ",nlpyrs
  NDays = (IYear-1)*365 + NLpYrs + DOY
  MJD = NDays + MD1900

! write(*,*) "iy, doy, mjd ",iy,doy,mjd

END SUBROUTINE DOY_MJD




!-MJD_DOY------------------------------------------------------------------
!
!  Subroutine to convert MJD to YMD
!
!  Input:
!    MJD - Modified Julian Date
!
!  Output:
!    IY - Year
!    DOY - Day of Year
!
!--------------------------------------------------------------------------

SUBROUTINE MJD_DOY( MJD, IY, DOY )

  INTEGER, INTENT(IN) :: MJD
  INTEGER, INTENT(OUT) :: IY, DOY
  INTEGER :: IDays, ILEAPY

  IDays = MJD - MD1900
  IY = ((IDays-1)*100)/Days_Per_Year+1

  DOY = IDays - (Days_Per_Year*(IY-1))/100
  ILEAPY = MOD(IY,4)
  ILEAPY = MIN(ILEAPY,1)+1
  IF( ILEAPY /= 1 .AND. DOY > 365 ) THEN
     IY = IY + 1
     DOY = DOY - 365
  END IF
  IF ( IY >= 100 ) IY = IY + IY1900
  IF ( IY < 100 ) IY = IY + IY1900
! write(*,*) "iy ",iy

! write(*,*) "mjd, iy, doy ",mjd, iy, doy

END SUBROUTINE MJD_DOY





!-MJD_Year-------------------------------------------------------------------
!
! Routine to get Jan 0, year of data start from mjd seconds
!
! INPUT/OUTPUT:
!     MJD_Secs        MJD seconds
!     Year            Jan 0, of year
!     MJD_Year_Secs   MJD seconds of Jan 0, of year
!
!--------------------------------------------------------------------------
SUBROUTINE MJD_Year( MJD_Secs, Year, MJD_Year_Secs )

 REAL(KIND = DB), INTENT(IN) :: MJD_Secs
 REAL(KIND = DB), INTENT(OUT) :: MJD_Year_Secs
 INTEGER, INTENT(OUT) :: Year

 INTEGER :: MJD, IDays, MJD_Year_Days, IYear, YMD
 REAL(KIND = DB), PARAMETER :: Zero = 0.0_DB
 REAL(KIND = DB) :: HMS, FDAY
 REAL(KIND = DB), PARAMETER :: OneD2 = 1.0E2_DB


 MJD = MJD_Secs/SecDay
! write(*,*) 'mjd 1',mjd
 IDays = MJD + MD1900 
 IYear = ((IDays*100)/Days_Per_Year) - 1
! write(*,*) 'iyear',iyear
 Year = IYear + IY1900 
 YMD = Year*10000+100
 HMS = Zero
 
 CALL YMDHMS_MJDDAY( YMD, HMS, MJD, FDAY )

 MJD_Year_Secs = (MJD-MJD_Offset) * SecDay

END SUBROUTINE MJD_Year




!-MJD_YMD------------------------------------------------------------------
!
!  Subroutine to convert MJD to YMD
!
!  Input:
!    MJD - Modified Julian Date
!
!  Output:
!    YMD - Year, month, day
!
!--------------------------------------------------------------------------

SUBROUTINE MJD_YMD( MJD, YMD )

  INTEGER, INTENT(IN) :: MJD
  INTEGER, INTENT(OUT) :: YMD

  INTEGER :: IDays, IYR, IYIND, Mnth, Day, IYR_Chk
  INTEGER :: I

  IDays = MJD - MD1900
  IYR = (IDays*100)/Days_Per_Year
! write(*,*) "iyr ",iyr

  IDays = IDays - (Days_Per_Year*IYR)/100

  IYR = IYR + 1
  IF( IDays == 0 ) THEN
    IYR = IYR - 1
    IDays = No_Days
  ENDIF

! write(*,*) " idays ",idays
! write(*,*) "iyr ", iyr
  IYIND = 1
  IF( MOD(IYR,4) == 0 .AND. MOD(IYR,400) /= 0) IYIND = 2

! write(*,*) " iyind ",iyind

  DO I = 2, 13 
     Mnth = I
     IF( Month(I,IYIND) >= IDays ) EXIT
  END DO

  IF ( I == 14 ) I = I-1
  Mnth = Mnth - 1
! write(*,*) "month ",i
  Day = IDays - Month(Mnth,IYIND)
! write(*,*) "day ",day
  IYR = IYR + IY1900
  YMD = 10000*IYR + 100*Mnth + Day

! write( *,*) "mjd, ymd = ", mjd,ymd
     
END SUBROUTINE MJD_YMD 




!-MJDDAY_YMDHMS------------------------------------------------------------
!
!  Subroutine to convert MJD, FDAY to YMDHMS
!
!  Input:
!    MJD - Modified Julian Date
!    FDAY - Fraction of a day
!
!  Output:
!    YMD - Year, month, day
!    HMS - Hour, minute, second
!
!--------------------------------------------------------------------------

SUBROUTINE MJDDAY_YMDHMS( MJD, FDAY, YMD, HMS )

  INTEGER, INTENT(IN) :: MJD
  REAL(KIND=DB), INTENT(IN) :: FDAY
  INTEGER, INTENT(OUT) :: YMD
  REAL(KIND=DB), INTENT(OUT) :: HMS

  INTEGER :: IDays, IYR, IYIND, Mnth, Day, I 
  INTEGER :: IHR, IMIN
  REAL(KIND = DB) :: HR, DMIN, SECS

  IDays = MJD - MD1900
  IYR = ( IDays * 100 )/Days_Per_Year
  IDays = IDays - ( IYR * Days_Per_Year)/100

  IYR = IYR + 1
  IF( IDays == 0 ) THEN
    IYR = IYR - 1
    IDays = No_Days
  ENDIF

! write(*,*) " idays ",idays
! write(*,*) "iyr ", iyr
  IYIND = 1
  IF( MOD(IYR,4) == 0 .AND. MOD(IYR,400) /= 0) IYIND = 2

! write(*,*) " iyind ",iyind

  DO I = 2, 13 
     Mnth = I
     IF( Month(I,IYIND) >= IDays ) EXIT
  END DO

  IF ( I == 14 ) I = I-1
  Mnth = Mnth - 1
! write(*,*) "month ",i
  Day = IDays - Month(Mnth,IYIND)
! write(*,*) "day ",day
  IYR = IYR + IY1900
  YMD = 10000*IYR + 100*Mnth + Day

  HR = FDAY * HourDay
  IHR = HR
  DMIN = (HR - IHR) * SecMin
  IMIN = DMIN
  SECS = (DMIN-IMIN) * SecMin

  HMS = DBLE(IHR) * 10000.0_DB +  DBLE(IMIN) * 100.0_DB + SECS

! write(*,*) "mjd fday ymd hms ", mjd,fday,ymd,hms

END SUBROUTINE MJDDAY_YMDHMS




!-MJDSEC_YMDHMS------------------------------------------------------------
!
!  Subroutine to convert MJD in Secs to YMDHMS
!
!  Input:
!    MJDSEC - Modified Julian Date in Seconds
!
!  Output:
!    YMD - Year, month, day
!    HMS - Hour, minute, second
!
!--------------------------------------------------------------------------

SUBROUTINE MJDSEC_YMDHMS( MJDSEC, YMD, HMS )

  REAL(KIND = DB), INTENT(IN) :: MJDSEC
  INTEGER, INTENT(OUT) :: YMD
  REAL(KIND = DB), INTENT(OUT) :: HMS

  INTEGER :: IDays, IYR, IYIND, Mnth, Day, I 
  INTEGER :: IHR, IMIN
  REAL(KIND = DB) :: HR, DMIN, SECS

  IDays = MJDSEC/SecDay + (MJD_Offset-MD1900)
  IYR = ( IDays * 100 )/Days_Per_Year
  IDays = IDays - ( IYR * Days_Per_Year)/100

  IYR = IYR + 1
  IF( IDays == 0 ) THEN
    IYR = IYR - 1
    IDays = No_Days
  ENDIF

  IYIND = 1
  IF( MOD(IYR,4) == 0 .AND. MOD(IYR,400) /= 0) IYIND = 2

  DO I = 2, 13 
     Mnth = I
     IF( Month(I,IYIND) >= IDays ) EXIT
  END DO

  IF ( I == 14 ) I = I-1
  Mnth = Mnth - 1
  !write(*,*) "month ",i
  Day = IDays - Month(Mnth,IYIND)
  !write(*,*) "day month",day,month(mnth,iyind)
  IYR = IYR + IY1900
  YMD = 10000*IYR + 100*Mnth + Day

  SECS = MOD( MJDSEC,SecDay )
  IMIN = SECS/SecMin
  SECS = MOD( SECS, SecMin)
  IHR = IMIN/INT(SecMin)
  IMIN = MOD( IMIN,INT(SecMin) )

  HMS = DBLE(IHR) * 10000.0_DB +  DBLE(IMIN) * 100.0_DB + SECS

  !write(*,*) "mjdsec ymd hms ", mjdsec,ymd,hms


END SUBROUTINE MJDSEC_YMDHMS


!-MJDSEC_FSEC_YMDHMS--------------------------------------------------------
!
!  Subroutine to convert MJD in Secs and Fractional Seconds to YMDHMS
!
!  Input:
!    MJDSEC - Modified Julian Date in Seconds
!    FSEC - Fractional Seconds
!
!  Output:
!    YMD - Year, month, day
!    HMS - Hour, minute, second
!
!--------------------------------------------------------------------------

SUBROUTINE MJDSEC_FSEC_YMDHMS( MJDSEC, FSEC, YMD, HMS )

  REAL(KIND = DB), INTENT(IN) :: MJDSEC, FSEC
  INTEGER, INTENT(OUT) :: YMD
  REAL(KIND = DB), INTENT(OUT) :: HMS

  INTEGER :: IDays, IYR, IYIND, Mnth, Day, I 
  INTEGER :: IHR, IMIN
  REAL(KIND = DB) :: HR, DMIN, SECS
  REAL(KIND = DB) :: MJDSEC_1, FSEC_1

  ! ensure fractional sec only contains fraction part
  MJDSEC_1 = MJDSEC + INT(FSEC)
  FSEC_1 = FSEC - INT(FSEC)

  IDays = MJDSEC_1/SecDay + (MJD_Offset-MD1900)
  IYR = ( IDays * 100 )/Days_Per_Year
  IDays = IDays - ( IYR * Days_Per_Year)/100

  IYR = IYR + 1
  IF( IDays == 0 ) THEN
    IYR = IYR - 1
    IDays = No_Days
  ENDIF

  IYIND = 1
  IF( MOD(IYR,4) == 0 .AND. MOD(IYR,400) /= 0) IYIND = 2

  DO I = 2, 13 
     Mnth = I
     IF( Month(I,IYIND) >= IDays ) EXIT
  END DO

  IF ( I == 14 ) I = I-1
  Mnth = Mnth - 1
  !write(*,*) "month ",i
  Day = IDays - Month(Mnth,IYIND)
  !write(*,*) "day month",day,month(mnth,iyind)
  IYR = IYR + IY1900
  YMD = 10000*IYR + 100*Mnth + Day

  SECS = MOD( MJDSEC_1,SecDay )
  IMIN = SECS/SecMin
  SECS = MOD( SECS, SecMin)
  IHR = IMIN/INT(SecMin)
  IMIN = MOD( IMIN,INT(SecMin) )
  SECS = SECS + FSEC_1

  HMS = DBLE(IHR) * 10000.0_DB +  DBLE(IMIN) * 100.0_DB + SECS

  !write(*,*) "mjdsec ymd hms ", mjdsec,ymd,hms


END SUBROUTINE MJDSEC_FSEC_YMDHMS


!-MJDSEC_FSEC_ADD-----------------------------------------------------------
!
!  Subroutine to add/subtract MJDSEC_2, FSEC_2 to/from MJDSEC_1, FSEC_1 
!  and return MJDSEC_SUM, FSEC_SUM
!
!  Input:
!    MJDSEC_1 - Modified Julian Date in Seconds
!    FSEC_1 - Fractional Seconds
!    MJDSEC_2 - Modified Julian Date in Seconds
!    FSEC_2 - Fractional Seconds
!
!  Output:
!    MJDSEC_SUM - Sum in Modified Julian Date Seconds
!    FSEC_SUM - Sum in Fractional Seconds
!    HMS - Hour, minute, second
!
!--------------------------------------------------------------------------

SUBROUTINE MJDSEC_FSEC_ADD( MJDSEC_1, FSEC_1, MJDSEC_2, FSEC_2, &
                        MJDSEC_SUM, FSEC_SUM)

  REAL(KIND = DB), INTENT(IN) :: MJDSEC_1, FSEC_1
  REAL(KIND = DB), INTENT(IN) :: MJDSEC_2, FSEC_2
  REAL(KIND = DB), INTENT(INOUT) :: MJDSEC_SUM, FSEC_SUM

  REAL(KIND = DB), PARAMETER :: ZERO = 0.0_DB
  REAL(KIND = DB) :: FSEC, MJDSEC, SECS

  FSEC = FSEC_1 + FSEC_2
  SECS = INT(FSEC)
  FSEC = FSEC - INT(FSEC)
  MJDSEC = MJDSEC_1 + SECS
  IF( FSEC < ZERO ) THEN
    MJDSEC = MJDSEC - 1.0_DB
    FSEC = FSEC + 1
  END IF
  FSEC_SUM = FSEC - INT(FSEC)

  MJDSEC_SUM = MJDSEC + MJDSEC_2 + INT(FSEC)

END SUBROUTINE MJDSEC_FSEC_ADD


!-TIME_ROUND---------------------------------------------------------------
!
!  Subroutine to round YMD HMS by the delta value
!
!  Input/Output:
!    YMD - Year, month, day
!    HMS - Hours, minutes, seconds
!
!  Input:
!    Delta - Delta value used for rounding
!
!--------------------------------------------------------------------------

SUBROUTINE TIME_ROUND( YMD, HMS, DELTA )

  INTEGER, INTENT(INOUT) :: YMD
  REAL(KIND = DB), INTENT(INOUT) :: HMS
  REAL(KIND = DB), INTENT(IN) :: DELTA

  REAL(KIND = DB), PARAMETER :: X6400 = 6400.0_DB, X40 = 40.0_DB
  REAL(KIND = DB) :: MJDSEC, TSEC
  INTEGER :: IHMS, IH, IMS, IM


  
  CALL YMDHMS_MJDSEC( YMD, HMS, MJDSEC )
  MJDSEC = MJDSEC + DELTA
  CALL MJDSEC_YMDHMS( MJDSEC, YMD, HMS )

END SUBROUTINE TIME_ROUND



!-YMD_MJD------------------------------------------------------------------
!
!  Subroutine to convert YMD to MJD
!
!  Input:
!    YMD - Year, month, day
!
!  Output:
!    MJD - Modified Julian Date
!
!--------------------------------------------------------------------------

SUBROUTINE YMD_MJD( YMD, MJD )

  INTEGER, INTENT(IN) :: YMD
  INTEGER, INTENT(OUT) :: MJD

  INTEGER :: IDOY, IYR, IYIND, Mnth, Day, MMDD, IYR_Chk

  IYR = YMD/10000
  MMDD = YMD - (IYR*10000)
  Mnth = MMDD/100
  Day = MOD(MMDD,100)
  ! write(*,*) "iyr, mnth, day ",iyr,mnth,day

  IYIND = 1
  IF( IYR == 0 ) THEN
    IYIND = 2
  END IF
  IF( MOD(IYR,4) == 0 .and. ( MOD(IYR,400) /= 0 .or. IYR == IY200) ) IYIND = 2
  IYR_Chk = IYR - IY1900
  IF( IYR_Chk < 0) THEN
     IF( IYR < 50 ) IYR = IYR + 100
  ELSE 
     IYR = IYR_Chk
  END IF

  ! write(*,*) "iyr month",iyr,Month(Mnth,iyind)
  IDOY = Month(Mnth,IYIND ) + Day
  ! write(*,*) " idoy ", idoy

  MJD = IDOY + ( ((IYR - 1) * Days_Per_Year) /100) + MD1900

! write(*,*) "YMD MJD = ", YMD,MJD
  

END SUBROUTINE YMD_MJD


!-YMDHMS_MJDDAY------------------------------------------------------------
!
!  Subroutine to convert YMDHMS to MJD, FDAY
!
!  Input:
!    YMD - Year, month, day
!    HMS - Hour, minute, second
!
!  Output:
!    MJD - Modified Julian Date
!    FDAY - Fraction of a day
!
!--------------------------------------------------------------------------

SUBROUTINE YMDHMS_MJDDAY( YMD, HMS, MJD, FDAY )

  INTEGER, INTENT(IN) :: YMD
  REAL(KIND=DB), INTENT(IN) :: HMS
  INTEGER, INTENT(OUT) :: MJD
  REAL(KIND=DB), INTENT(OUT) :: FDAY

  INTEGER :: IYR, IMO, IDAY, MMDD, DOY, IYIND, IYR_Chk
  INTEGER :: IHMS, IHR, IMSEC, IMIN, ISEC
  REAL(KIND=DB) :: FSEC, TIME

  IYR = YMD/10000
  MMDD = YMD - (IYR*10000)
  IMO = MMDD/100
  IDAY = MOD(MMDD,100)

  IYIND = 1
  IF( IYR == 0 ) THEN
    IYIND = 2
  END IF
  IF( MOD(IYR,4) == 0 .and. ( MOD(IYR,400) /= 0 .or. IYR == IY200) ) IYIND = 2

  IYR_Chk = IYR - IY1900
  IF( IYR_Chk < 0) THEN
     IF( IYR < 50 ) IYR = IYR + 100
  ELSE 
     IYR = IYR_Chk
  END IF
  
  DOY = MONTH( IMO, IYIND) + IDAY

  MJD = DOY + ( ((IYR-1) * 36525)/100 ) + MD1900

  IHMS = HMS
  FSEC = HMS - IHMS
  IHR = IHMS/10000
  IMSEC = IHMS - (IHR*10000)
  IMIN = IMSEC/100
  ISEC = MOD( IMSEC,100 )

  TIME = DBLE(IHR) * SecHour + DBLE(IMIN) * SecMin + DBLE(ISEC) + FSEC

  FDAY = TIME/SecDay

! write(*,*) "YMD HMS MJD FDAY ",YMD,HMS,MJD,FDAY


END SUBROUTINE YMDHMS_MJDDAY



!-YMDHMS_MJDSEC------------------------------------------------------------
!
!  Subroutine to convert YMDHMS to MJD in Seconds
!
!  Input:
!    YMD - Year, month, day
!    HMS - Hour, minute, second
!
!  Output:
!    MJDSEC - Modified Julian Date in Seconds
!
!--------------------------------------------------------------------------

SUBROUTINE YMDHMS_MJDSEC( YMD, HMS, MJDSEC )

  INTEGER, INTENT(IN) :: YMD
  REAL(KIND=DB), INTENT(IN) :: HMS
  REAL(KIND=DB), INTENT(OUT) :: MJDSEC

  INTEGER :: IYR, IMO, IDAY, MMDD, DOY, IYIND, IYR_Chk
  INTEGER :: IHMS, IHR, IMSEC, IMIN, ISEC
  INTEGER :: MJD
  REAL(KIND=DB) :: FSEC, TIME

  IYR = YMD/10000
  MMDD = YMD - (IYR*10000)
  IMO = MMDD/100
  IDAY = MOD(MMDD,100)

  IYIND = 1
  IF( IYR == 0 ) THEN
    IYIND = 2
  END IF
  IF( MOD(IYR,4) == 0 .and. ( MOD(IYR,400) /= 0 .or. IYR == IY200) ) IYIND = 2

  IYR_Chk = IYR - IY1900
  IF( IYR_Chk < 0) THEN
     IF( IYR < 50 ) IYR = IYR + 100
  ELSE 
     IYR = IYR_Chk
  END IF
  
  DOY = MONTH( IMO, IYIND) + IDAY

  MJD = DOY + ( ((IYR-1) * 36525)/100 ) - (MJD_Offset - MD1900)
  IHMS = HMS
  FSEC = HMS - IHMS
  IHR = IHMS/10000
  IMSEC = IHMS - (IHR*10000)
  IMIN = IMSEC/100
  ISEC = MOD( IMSEC,100 )

  TIME = DBLE(IHR) * SecHour + DBLE(IMIN) * SecMin + DBLE(ISEC) + FSEC

  MJDSEC =  MJD*SecDay + TIME

! write(*,*) "YMD HMS MJDSEC ",YMD,HMS,MJDSEC


END SUBROUTINE YMDHMS_MJDSEC


!-YMDHMS_MJDSEC_FSEC-------------------------------------------------------
!
!  Subroutine to convert YMDHMS to MJD in Seconds and Fractional Seconds
!
!  Input:
!    YMD - Year, month, day
!    HMS - Hour, minute, second
!
!  Output:
!    MJDSEC - Modified Julian Date in Seconds
!    FSEC - Fractional Seconds
!
!--------------------------------------------------------------------------

SUBROUTINE YMDHMS_MJDSEC_FSEC( YMD, HMS, MJDSEC, FSEC )

  INTEGER, INTENT(IN) :: YMD
  REAL(KIND=DB), INTENT(IN) :: HMS
  REAL(KIND=DB), INTENT(OUT) :: MJDSEC, FSEC

  INTEGER :: IYR, IMO, IDAY, MMDD, DOY, IYIND, IYR_Chk
  INTEGER :: IHMS, IHR, IMSEC, IMIN, ISEC
  INTEGER :: MJD
  REAL(KIND=DB) :: TIME

  IYR = YMD/10000
  MMDD = YMD - (IYR*10000)
  IMO = MMDD/100
  IDAY = MOD(MMDD,100)

  IYIND = 1
  IF( IYR == 0 ) THEN
    IYIND = 2
  END IF
  IF( MOD(IYR,4) == 0 .and. ( MOD(IYR,400) /= 0 .or. IYR == IY200) ) IYIND = 2

  IYR_Chk = IYR - IY1900
  IF( IYR_Chk < 0) THEN
     IF( IYR < 50 ) IYR = IYR + 100
  ELSE 
     IYR = IYR_Chk
  END IF
  
  DOY = MONTH( IMO, IYIND) + IDAY

  MJD = DOY + ( ((IYR-1) * 36525)/100 ) - (MJD_Offset - MD1900)
  FSEC = HMS - INT(HMS)
  IHR = INT(HMS)/10000
  IMSEC = INT(HMS) - (IHR*10000)
  IMIN = IMSEC/100
  ISEC = MOD( IMSEC,100 )

  TIME = DBLE(IHR) * SecHour + DBLE(IMIN) * SecMin + DBLE(ISEC) 

  MJDSEC =  MJD*SecDay + TIME

! write(*,*) "YMD HMS MJDSEC FSEC",YMD,HMS,MJDSEC,FSEC


END SUBROUTINE YMDHMS_MJDSEC_FSEC


!-GPSWEEK_DOY------------------------------------------------------------- 
!
!  Subroutine to calculate the the day of year given the gps week 
!
!  Input:
!   GPSWEEK - GPS week  
!
!  Output:
!   IY  - Year
!   DOY - Day of Year
!
! NOTE:
!    from internet discussing GPS Week Rollover
!  The GPS Week Number count began at approximately midnight on the evening of
!  05 January 1980 / morning of 06 January 1980. Since that time, the count has
!  been incremented by 1 each week, and broadcast as part of the GPS
!  message. The GPS Week Number field is modulo 1024. This means that at the
!  completeion of week 1023, the GPS week number will rollover to 0 on midnight
!  of the evening of 21 August 1999 / morning of 22 August 1999.
!
!--------------------------------------------------------------------------
SUBROUTINE GPSWEEK_DOY( GPS_Week, IY, DOY)

 INTEGER, INTENT(IN) :: GPS_Week
 INTEGER, INTENT(OUT) :: IY, DOY

 REAL(KIND=DB), PARAMETER :: GPS_Modulo = 1024.0_DB
 REAL(KIND=DB), PARAMETER ::  &
                  Start_GPS_Mjdsecs = 1230681600.0_DB    ! 01/06/99 00:00:00
 REAL(KIND=DB), PARAMETER :: NDays_Week  = 7.0_DB
 REAL(KIND=DB) :: Secs, Past_Days, Mjdsecs_Date
 INTEGER :: MJD

 ! change gpsweek to the number of days
 Past_Days = GPS_Week * NDays_Week
 Secs =  Past_Days * SecDay

 ! add in gps start secs
 Mjdsecs_Date = Secs + Start_GPS_Mjdsecs

 ! Convert back to days
 Past_Days = Mjdsecs_Date/SecDay
 
 ! add in mjd_offset
 MJD = Past_Days + MJD_Offset

 ! convert MJD to DOY
 CALL MJD_DOY( MJD, IY, DOY )

END SUBROUTINE GPSWEEK_DOY


!-YMDHMS_GPSWEEK---------------------------------------------------------- 
!
!  Subroutine to calculate the gps week given a date and time
!
!  Input:
!   YMD - Year, Month, Day
!   HMS - Hours, Minutes, Seconds
!
!  Output:
!   GPSWEEK - GPS week  
!   Day_of_Week - Day within week where 0 = Sunday and 6 = Saturday
!
! NOTE:
!    from internet discussing GPS Week Rollover
!  The GPS Week Number count began at approximately midnight on the evening of
!  05 January 1980 / morning of 06 January 1980. Since that time, the count has
!  been incremented by 1 each week, and broadcast as part of the GPS
!  message. The GPS Week Number field is modulo 1024. This means that at the
!  completeion of week 1023, the GPS week number will rollover to 0 on midnight
!  of the evening of 21 August 1999 / morning of 22 August 1999.
!
!--------------------------------------------------------------------------
SUBROUTINE YMDHMS_GPSWEEK( YMD, HMS, GPS_Week, Day_of_Week )

 INTEGER, INTENT(IN) :: YMD
 REAL(KIND=DB), INTENT(IN) :: HMS
 INTEGER, INTENT(OUT) :: GPS_Week, Day_of_Week

 REAL(KIND=DB), PARAMETER :: GPS_Modulo = 1024.0_DB
 REAL(KIND=DB), PARAMETER ::  &
                  Start_GPS_Mjdsecs = 1230681600.0_DB    ! 01/06/99 00:00:00
 REAL(KIND=DB), PARAMETER :: NDays_Week  = 7.0_DB
 REAL(KIND=DB) :: Mjdsecs_Date, Past_Days, Week_Number
 INTEGER :: Num_Week

 ! convert date and time to mjdsecs
 CALL YMDHMS_MJDSEC( YMD, HMS, Mjdsecs_Date )

 ! compute number of gps weeks from 01/06/99 00:00:00 
 Past_Days = (Mjdsecs_Date - Start_GPS_Mjdsecs)/SecDay
 
 Week_Number =  Past_Days/NDays_Week
 Num_Week = Week_Number
 Day_of_Week = Past_Days - Num_Week*NDays_Week
 ! GPS week modulo 1024 is not used for many gps week computations
 ! therefore, this will be commented here and done outside the program.
 !GPS_Week = MOD(Week_Number, GPS_Modulo)
 GPS_Week = Week_Number

END SUBROUTINE YMDHMS_GPSWEEK



!-YRDOY_GPSWEEK----------------------------------------------------------- 
!
!  Subroutine to calculate the gps week given year and day of year
!
!  Input:
!   YMD - Year, Month, Day
!   HMS - Hours, Minutes, Seconds
!
!  Output:
!   GPSWEEK - GPS week  
!   Day_of_Week - Day within week where 0 = Sunday and 6 = Saturday
!
! NOTE:
!    from internet discussing GPS Week Rollover
!  The GPS Week Number count began at approximately midnight on the evening of
!  05 January 1980 / morning of 06 January 1980. Since that time, the count has
!  been incremented by 1 each week, and broadcast as part of the GPS
!  message. The GPS Week Number field is modulo 1024. This means that at the
!  completeion of week 1023, the GPS week number will rollover to 0 on midnight
!  of the evening of 21 August 1999 / morning of 22 August 1999.
!
!--------------------------------------------------------------------------
SUBROUTINE YRDOY_GPSWEEK( YrDoy, GPS_Week, Day_of_Week )

 INTEGER, INTENT(IN) :: YrDoy
 INTEGER, INTENT(OUT) :: GPS_Week, Day_of_Week

 INTEGER :: Year, YMD, Doy
 REAL(KIND=DB), PARAMETER :: GPS_Modulo = 1024.0_DB
 REAL(KIND=DB), PARAMETER ::  &
                  Start_GPS_Mjdsecs = 1230681600.0_DB    ! 01/06/99 00:00:00
 REAL(KIND=DB), PARAMETER :: NDays_Week  = 7.0_DB
 REAL(KIND=DB) :: HMS, Mjdsecs_Date, Past_Days, Week_Number
 REAL(KIND = DB), PARAMETER :: Zero = 0.0_DB
 INTEGER :: Num_Week

 Year = YrDoy/1000
 YMD = Year * 10000+100+1
 Doy = YrDoy - Year*1000
 HMS = Zero
 ! convert date and time to mjdsecs
 CALL YMDHMS_MJDSEC( YMD, HMS, Mjdsecs_Date )

 ! compute number of gps weeks from 01/06/99 00:00:00 
 Past_Days = (Mjdsecs_Date - Start_GPS_Mjdsecs)/SecDay

 Past_Days = Past_Days + Doy
 
 Week_Number =  Past_Days/NDays_Week
 Num_Week = Week_Number
 Day_of_Week = Past_Days - Num_Week*NDays_Week

 ! GPS week modulo 1024 is not used for many gps week computations
 ! therefore, this will be commented here and done outside the program.
 !GPS_Week = MOD(Week_Number, GPS_Modulo)
 GPS_Week = Week_Number

END SUBROUTINE YRDOY_GPSWEEK





!-MJM_MJD------------------------------------------------------------------
!
!  Subroutine to convert from VMSI MJM to MJD
!
!  Input:
!   MJM - modified Julian minute
!
!  Output:
!    MJD - Modified Julian Date
!
!--------------------------------------------------------------------------

SUBROUTINE MJM_MJD( MJM, MJD, DAY_FRAC )


  INTEGER, INTENT(IN) ::  MJM
  INTEGER, INTENT(OUT) :: MJD
  REAL(KIND = DB) :: DAY_FRAC


  INTEGER :: IYear, NLpYrs, NDays

  MJD = MJM / IMIN_DAY + MJD_Offset
  DAY_FRAC = mod( dble( MJM ), XMIN_DAY ) /XMIN_DAY


!  write(*,*) 'MJM, MJD, DAY_FRAC ',MJM, MJD, DAY_FRAC

END SUBROUTINE MJM_MJD




!-MJD_MJM------------------------------------------------------------------
!
!  Subroutine to convert from MJD and DAY_FRAC to VMSI MJM 
!
!  Input:
!    MJD - Modified Julian Date
!    DAY_FRAC - fractional day
!
!  Output:
!   MJM - modified Julian minute
!
!--------------------------------------------------------------------------

SUBROUTINE MJD_MJM( MJD, DAY_FRAC, MJM )

  REAL(KIND = DB), PARAMETER :: OneD2 = 1.0E2_DB

  INTEGER, INTENT(IN) ::  MJD
  INTEGER, INTENT(OUT) :: MJM
  REAL(KIND = DB), INTENT(IN)  :: DAY_FRAC




  MJM = (MJD -  MJD_Offset) * IMIN_DAY
  MJM =  MJM + INT(DAY_FRAC * XMIN_DAY + OneD2 )


!  write(*,*) 'MJM, MJD, DAY_FRAC ',MJM, MJD, DAY_FRAC

END SUBROUTINE MJD_MJM




!-MJM_YMDHMS------------------------------------------------------------------
!
!  Subroutine to convert from VMSI MJM to YMD HMS
!
!  Input:
!   MJM - modified Julian minute
!
!  Output:
!    YMD
!    HMS
!
!--------------------------------------------------------------------------

SUBROUTINE MJM_YMDHMS( MJM, YMD, HMS  )


  INTEGER, INTENT(IN) ::  MJM
  INTEGER  :: MJD
  INTEGER :: YMD
  REAL(KIND = DB) :: HMS

  REAL(KIND = DB) :: DAY_FRAC



  call MJM_MJD( MJM, MJD, DAY_FRAC )

  call MJDDAY_YMDHMS( MJD, DAY_FRAC , YMD, HMS )


!  write(*,*) 'MJM, MJD, DAY_FRAC ',MJM, MJD, DAY_FRAC
!  write(*,*) 'YMD, HMS ', YMD, HMS

END SUBROUTINE MJM_YMDHMS





!-MJM_date------------------------------------------------------------------
!
!  Subroutine to convert from VMSI MJM to new date type
!
!  Input:
!   MJM - modified Julian minute
!
!  Output:
!    
!    new_date - of type ymd_date = ( yymmdd, hhmm, sec )
!
!--------------------------------------------------------------------------

SUBROUTINE MJM_date( MJM, new_date  )

!!  use new_types_module 


  type ( ymd_date )  :: new_date

  INTEGER, INTENT(IN) ::  MJM
  INTEGER :: YMD
  REAL(KIND = DB) :: HMS

  REAL(KIND = DB) :: DAY_FRAC



  call MJM_YMDHMS( MJM, YMD, HMS )

        new_date%yymmdd = YMD
        new_date%hhmm   = INT( HMS / 100.0_DB + 0.01_DB )
        new_date%sec    = mod( HMS , 100.0_DB )




!  write(*,*) 'MJM, YMD, HMS ', MJM, YMD, HMS
!  write(*,*) 'new_date%yymmdd ', new_date%yymmdd
!  write(*,*) 'new_date%hhmm ', new_date%hhmm
!  write(*,*) 'new_date%sec ', new_date%sec

END SUBROUTINE MJM_date


!-SET_TIME_REF-------------------------------------------------------------
!
!  Subroutine to set the geodyn time reference
!
!  Input:
!   Reference - Geodyn Reference Time
!
!  Output:
!    MJD_Offset - Modified Geodyn Reference Offset
!
!--------------------------------------------------------------------------

SUBROUTINE SET_TIME_REF( Geodyn_Reference )

  REAL(KIND=DB), INTENT(IN) :: Geodyn_Reference

  REAL(KIND = DB), PARAMETER :: Zero = 0.0_DB

  IF( Geodyn_Reference > Zero ) THEN
     MJD_Offset = INT(Geodyn_Reference - JD_Offset + .00001); 
  END IF

END SUBROUTINE SET_TIME_REF


END MODULE Date_Time_Module

