PROGRAM bih_master_update
!--------------------------------------------------------------------------
!
!  PURPOSE:  Update the bih ascii master tables file with all current
!            information.
!
!
!  INPUT
!    master_bih.table         master bih table in ascii format
!    UTC-TAI.history          history of UTC-TAI data
!    eopc04_IAU2000.62-now    current pole values
!    finals2000A.daily        predicted pole values
!    ccpp                     previous year's flux values
!    ccyy                     current year's flux values
!    DGD.txt                  estimated temporary daily geomagnetic data
!    DSD.txt                  estimated daily solar data
!    45DF.txt                 45-Day AP and F10.7cm Flux prediction
!    ig_rz.dat                Ionosphere data
!  
!   modified 11/8/2007 to read new eopc04.62-now format
!
!   Jan 10, 2012 - eopc04_IAU2000.62-now replaces eopc04.62-now and
!       finals2000A.daily replaces ser7.dat
!    
!--------------------------------------------------------------------------
!
! define Modules
  USE kinds_mod
  USE date_time_module
  USE global_module

 IMPLICIT NONE

 CHARACTER(80) :: Aline
 CHARACTER(10) :: today_date

 LOGICAL :: LEXIST

! Open log output file
 OPEN(UNIT = Error_UnitNum, FILE = "master_bih_update.log", FORM = "FORMATTED",&
      STATUS = "REPLACE", IOSTAT = OpenStatus )
 IF (OpenStatus > 0) THEN
    WRITE( *,* ) "ERROR *** Cannot open error output file."
    STOP
 END IF

! Open report output file
 OPEN(UNIT = Report_UnitNum, FILE = report_file, FORM = "FORMATTED",&
      STATUS = "UNKNOWN", ACTION = "READWRITE", POSITION = "APPEND",&
      IOSTAT = OpenStatus )
 IF (OpenStatus > 0) THEN
    WRITE( *,* ) "ERROR *** Cannot open output file, ", report_file
    STOP
 END IF

! Open a1utc table output file
 OPEN(UNIT = Table_UnitNum, FILE = table_file, FORM = "FORMATTED",&
      STATUS = "UNKNOWN", ACTION = "READWRITE", IOSTAT = OpenStatus )
 IF (OpenStatus > 0) THEN
    WRITE( *,* ) "ERROR *** Cannot open output file, ", table_file
    STOP
 END IF

! Open new master bih file
 OPEN(UNIT = NEWBIH_UnitNum, FILE = new_bih_file, FORM = "FORMATTED",&
      STATUS = "NEW", IOSTAT = OpenStatus )
 IF (OpenStatus > 0) THEN
    WRITE( Error_UnitNum,* ) "ERROR *** Cannot open ",new_bih_file," output file."
    STOP
 END IF

! Open program input files
! inquire if master file in current directory and open
  INQUIRE( FILE = old_bih_file, EXIST = LEXIST )
  IF( LEXIST ) THEN
     OPEN(UNIT = OLDBIH_UnitNum, FILE = old_bih_file, FORM = "FORMATTED",&
         STATUS = "OLD", IOSTAT = OpenStatus )
     IF (OpenStatus > 0) THEN
       WRITE( Error_UnitNum,* ) "ERROR *** Cannot open input file ", old_bih_file
       STOP 16
     END IF
  ELSE
    WRITE(Error_UnitNum,  &
     "(/'ERROR: The master bih table, ', A,', in ascii format',&
     & /,'       does not exist in current directory.')") old_bih_file
     STOP 16
  END IF

! position old bih table after header and write header to new file
  READ( OLDBIH_UnitNum, "(A)", IOSTAT = InputStatus ) Aline
  IF (InputStatus /= 0) THEN
    WRITE( Error_UnitNum, * ) "ERROR *** Input file error reading ", old_bih_file
    STOP 16
  END IF
  IF( Aline(1:11) /= "MASTER FILE" ) THEN 
    WRITE( Error_UnitNum, * ) "ERROR *** Problem with format of ", old_bih_file
    WRITE( Error_UnitNum, * ) "ERROR *** Check file."
    STOP 16
  END IF
  WRITE(NEWBIH_UnitNum,"(A)") TRIM(Aline)
  
  ! Write report title and version number
  WRITE(Report_UnitNum, "(/'GEODYN BIH TABLE REPORT:'//)")  
  WRITE(Report_UnitNum, "('VERSION: ',F10.2)") Version
  CALL DATE_AND_TIME( today_date )

  WRITE(Report_UnitNum,  &
   "(/'  The GEODYN master bih table date of update:  ',A8)") today_date
  WRITE(Report_UnitNum, *) " A summary of the bih table section updates and/or predictions follows."
  WRITE(Report_UnitNum, *)



  ! A1UTC section
  CALL A1UTC( bih_sections(1) ) 

  ! POLE section
  CALL POLE( bih_sections(2) )

  ! FLUXS, FLUXAP, FLUXKP8 sections
  CALL SOLAR_FLUX( bih_sections(3), bih_sections(4), bih_sections(5), &
                   today_date )

  ! IONIND section
  CALL IONIND( bih_sections(6) )

 ! close files
 CLOSE( Report_UnitNum )
 CLOSE( Error_UnitNum )
 CLOSE( Table_UnitNum )
 CLOSE( NEWBIH_UnitNum )
 CLOSE( OLDBIH_UnitNum )


CONTAINS



!****************************************************************************
  SUBROUTINE A1UTC( a1utc_section )

  CHARACTER(7), INTENT(IN) :: a1utc_section

  CHARACTER(80) :: Aline

  INTEGER(KIND=i4b) :: Num_Records, UnitNum_Out, year, ymd_bih, hms_bih
  INTEGER(KIND=i4b) :: seq_no

  LOGICAL :: LUPDATE, LWRITE


  !write(*,*) 'a1utc_section ',a1utc_section

  INQUIRE( FILE = utctai_file, EXIST = LUPDATE )


  WRITE(Report_UnitNum,  &
    "('********************************************************************************')")
  WRITE(Report_UnitNum, "(A//)")  a1utc_section

  ! SCRATCH FILE
  OPEN(UNIT = Scratch_UnitNum, FORM = "FORMATTED", STATUS = "SCRATCH", &
     IOSTAT = OpenStatus )
  IF (OpenStatus > 0) THEN
     WRITE( Error_UnitNum,* ) "ERROR *** Cannot open scratch file."
     STOP
  END IF
  LWRITE = .FALSE.
  UnitNum_Out = Scratch_UnitNum

  CALL Section_Header( a1utc_section, Num_Records, LWRITE )
  CALL Transfer_Master_Section( OLDBIH_UnitNum, UnitNum_Out, Num_Records )

  IF( LUPDATE ) THEN
    WRITE(Report_UnitNum, *) "   The file, ",utctai_file, ", contains the history of the UTC-TA1 data"
    ! back up file 1 space to get date of last value
    BACKSPACE( Scratch_UnitNum )
    READ( Scratch_UnitNum, "(A)" ) Aline
    READ(Aline,"(I6,I6,T73,I8)") ymd_bih, hms_bih, seq_no
    year = ymd_bih/10000
    if(year < 50) then
      ymd_bih = 20000000 + ymd_bih
    else
      ymd_bih = 19000000 + ymd_bih
    end if
    !write(*,*) 'ymd_bih ',ymd_bih, hms_bih


    ! Read utc-tai history file to see if any updates exist
    CALL Update_A1UTC( Scratch_UnitNum, ymd_bih, hms_bih, seq_no )
    Num_Records = seq_no
     
  ELSE
    WRITE(Report_UnitNum, *) "   -------------------------------------------------------------------" 
    WRITE(Report_UnitNum, *) "   The file ", utctai_file, " does not exist in the current directory." 
    WRITE(Report_UnitNum, *) "   The ", TRIM(a1utc_section), " section will not be updated in the &
      &master bih table."
    WRITE(Report_UnitNum, *) "   -------------------------------------------------------------------" 
    WRITE(Report_UnitNum, "(//)") 
  END IF

  ! need to transfer section twice so we can build a1utc table
  REWIND( Scratch_UnitNum )
  ! write new section header with updated num of records
  CALL Update_Header( a1utc_section, Num_Records )
   
  ! write updated scratch file to new bih table
  CALL Transfer_Master_Section( Scratch_UnitNum, NEWBIH_UnitNum, Num_Records )

  REWIND( Scratch_UnitNum )
  CALL Create_A1UTC_Table( Scratch_UnitNum, Num_Records )
  CLOSE( Scratch_UnitNum )

  WRITE(Report_UnitNum,  &
    "('********************************************************************************'//)")

  END SUBROUTINE A1UTC



!****************************************************************************
  REAL(KIND=r8b) FUNCTION a1utc_for_mjd( mjd, mjdday_a1utc, a1utc_rate )

  INTEGER(KIND=i4b), INTENT(IN) :: mjd
  REAL(KIND=r8b), INTENT(INOUT) :: a1utc_rate, mjdday_a1utc

  REAL(KIND=r8b) :: dmjd, mjdday_table_curr, mjdday_table_prev
  REAL(KIND=r8b) :: a1utc_prev, a1utc_curr, a1utc_rate_prev, a1utc_rate_curr

  REWIND(Table_UnitNum)
  
  a1utc_for_mjd = zero
  a1utc_rate = zero
  mjdday_a1utc =  zero
  dmjd = float(mjd)

  READ(Table_UnitNum,"(D25.16,1X,D16.9,1X,D15.7)") mjdday_table_prev, a1utc_prev, a1utc_rate_prev
  IF( dmjd <= mjdday_table_prev ) THEN 
      a1utc_for_mjd = a1utc_prev
      a1utc_rate = a1utc_rate_prev
      mjdday_a1utc = mjdday_table_prev
      RETURN
  END IF

  DO
     READ( Table_UnitNum, "(A)", IOSTAT = InputStatus ) Aline
     IF( InputStatus < 0 ) THEN
       EXIT
     END IF
     IF (InputStatus /= 0) THEN
       WRITE( Error_UnitNum, * ) "ERROR *** Input file error reading ",table_file
       STOP 16
     END IF

     READ(Aline,"(D25.16,1X,D16.9,1X,D15.7)") mjdday_table_curr, a1utc_curr, a1utc_rate_curr

     IF( dmjd >= mjdday_table_prev .AND. dmjd < mjdday_table_curr ) THEN
        a1utc_for_mjd =  a1utc_prev
        a1utc_rate = a1utc_rate_prev
        mjdday_a1utc = mjdday_table_prev
        EXIT
     END IF

     a1utc_prev = a1utc_curr
     a1utc_rate_prev = a1utc_rate_curr
     mjdday_table_prev = mjdday_table_curr

  END DO

  IF( a1utc_for_mjd == 0 ) THEN
     a1utc_for_mjd = a1utc_curr
     a1utc_rate = a1utc_rate_curr
     mjdday_a1utc = mjdday_table_curr
  END IF

  END FUNCTION a1utc_for_mjd


!****************************************************************************
  SUBROUTINE Calculate_a1ut1( mjd, ut1utc, a1ut1 )

  INTEGER(KIND=i4b), INTENT(IN) :: mjd
  REAL(KIND=r8b), INTENT(IN) :: ut1utc
  REAL(KIND=r8b), INTENT(INOUT) :: a1ut1

  REAL(KIND=r8b) :: mjdday_a1utc
  REAL(KIND=r8b) :: xmjd, dpsi_out, deps_out
  REAL(KIND=r8b) :: a1utc, dut1, a1utc_rate
  REAL(KIND=r8b) :: jd

  REAL(KIND=r8b), PARAMETER :: tolsec = 0.10D-7

  a1utc = a1utc_for_mjd( mjd, mjdday_a1utc, a1utc_rate )
   
  IF( a1utc_rate > tolsec ) THEN
     a1ut1 = (a1utc + (a1utc_rate*(float(mjd) - mjdday_a1utc))) - ut1utc   
                                         !((a1-utc) + (a1utc_rate*delta_t) - ut1utc

  !print *,''
  !print *,'calculut1: mjd, mjdday_a1utc: ',mjd,mjdday_a1utc
  !print *,'calculut1: a1utc, a1utc_rate: ',a1utc,a1utc_rate
  !print *,'calculut1: ut1utc, a1ut1: ',ut1utc,a1ut1

  ELSE   
     a1ut1 = a1utc - ut1utc              ! (a1-utc) - (ut1-utc)
  END IF

  xmjd = mjd

  dut1 = UTTIDE(xmjd)                 !   dut1r = (ut1-ut1r)
  a1ut1 = a1ut1 + dut1                !   a.1-ut1r = (a.1-ut1) + (ut1-ut1r)
 
  !print *,'calculut1: dut1, a1ut1: ',dut1,a1ut1

 END SUBROUTINE Calculate_a1ut1 



!****************************************************************************
  SUBROUTINE Create_A1UTC_Table( UnitNum, Num_Records )

  INTEGER(KIND=i4b), INTENT(IN) :: UnitNum, Num_Records

  CHARACTER(80) :: Aline

  INTEGER(KIND=i4b) :: I, iymd, hh, mm, ss, mjd

  REAL(KIND=r8b) :: mjdsec, mjdday, total_seconds, secday
  REAL(KIND=r8b) :: a1utc_secs, a1utc_rate

  DO I = 1, Num_Records
    READ(UnitNum,"(I6,I2,I2,I2,2X,D16.9,D15.7)") iymd, hh, mm, ss, a1utc_secs, a1utc_rate
    CALL YMD_MJD(iymd, mjd)
    total_seconds = float(hh)*3600.0D0 + float(mm)*60.0D0 + float(ss)
    secday = total_seconds/secs_per_day
    mjdday =  float(mjd) + secday
    WRITE(Table_UnitNum,"(D25.16,1X,F16.11,1X,D15.7)") mjdday, a1utc_secs, a1utc_rate
  END DO

  END SUBROUTINE Create_A1UTC_Table


!****************************************************************************
  SUBROUTINE IONIND( iono_section )

  CHARACTER(7), INTENT(IN) :: iono_section

  CHARACTER(80) :: Aline

  INTEGER(KIND=i4b) :: Num_Records, UnitNum_In, UnitNum_Out
  INTEGER(KIND=i4b) :: seq_no

  LOGICAL :: LUPDATE, LWRITE


  !write(*,*) 'iono_section ',iono_section

  INQUIRE( FILE = iono_file, EXIST = LUPDATE )

  ! CHECK TO SEE IF IONIND SECTION EXISTS IN CURRENT TABLE
  READ(OLDBIH_UnitNum,"(A)", IOSTAT = InputStatus)
  IF (InputStatus < 0) THEN
    RETURN
  ELSE
    BACKSPACE(OLDBIH_UnitNum)
  END IF

  WRITE(Report_UnitNum,  &
    "('********************************************************************************')")
  WRITE(Report_UnitNum, "(A/)")  iono_section
  WRITE(Report_UnitNum, "(/' Current Ionosphere Values:')")

  IF( LUPDATE ) THEN
     WRITE(Report_UnitNum, *) "   The file, ",iono_file, ", contains the solar and ionospheric &
         &indices (IG12, Rz12)."

     ! SCRATCH FILE
     OPEN(UNIT = Scratch_UnitNum, FORM = "FORMATTED", STATUS = "SCRATCH", &
        IOSTAT = OpenStatus )
     IF (OpenStatus > 0) THEN
        WRITE( Error_UnitNum,* ) "ERROR *** Cannot open scratch file."
        STOP
     END IF

     ! SCRATCH2 FILE
     OPEN(UNIT = Scratch2_UnitNum, FORM = "FORMATTED", STATUS = "SCRATCH", &
        IOSTAT = OpenStatus )
     IF (OpenStatus > 0) THEN
        WRITE( Error_UnitNum,* ) "ERROR *** Cannot open scratch file."
        STOP
     END IF

     UnitNum_Out = Scratch2_UnitNum
     UnitNum_In  = Scratch_UnitNum
     LWRITE = .FALSE.

     CALL Section_Header( iono_section, Num_Records, LWRITE )
     CALL Transfer_Master_Section( OLDBIH_UnitNum, Scratch_UnitNum, Num_Records )

     ! update data
     CALL Update_Iono( UnitNum_In, UnitNum_Out, Num_Records, seq_no )
     
     ! write new section header with updated num of records
     CALL Update_Header( iono_section, seq_no )
     REWIND( UnitNum_Out )
   
     ! write updated scratch file to new bih table
     CALL Transfer_Master_Section( UnitNum_Out, NEWBIH_UnitNum, seq_no )

     CLOSE( Scratch_UnitNum )
 
  ELSE
     WRITE(Report_UnitNum, *) 
     WRITE(Report_UnitNum, *) "   -----------------------------------------------------------------"
     WRITE(Report_UnitNum, *) "   The file, ", iono_file, ", does not exist in the current directory." 
     WRITE(Report_UnitNum, *) "   The ", TRIM(iono_section), " section will not be updated in the &
       &master bih table."
     WRITE(Report_UnitNum, *) "   -----------------------------------------------------------------"
     LWRITE = .TRUE.
     UnitNum_Out = NEWBIH_UnitNum

     CALL Section_Header( iono_section, Num_Records, LWRITE )
     CALL Transfer_Master_Section( OLDBIH_UnitNum, NEWBIH_UnitNum, Num_Records )
  END IF

 
  WRITE(Report_UnitNum,  &
    "(/'********************************************************************************'//)")

  END SUBROUTINE IONIND



!****************************************************************************
  SUBROUTINE Output_Pole_Predicts( UnitNum, yy, mm, dd, mjd, xin, yin, &
                                ut1utc, dlod, dX, dY, seq_no )

  INTEGER(KIND=i4b), INTENT(IN) :: UnitNum
  CHARACTER(2), INTENT(INOUT) :: yy, mm, dd
  INTEGER(KIND=i4b), INTENT(IN) ::  mjd
  INTEGER(KIND=i4b), INTENT(INOUT) :: seq_no
  REAL(KIND=r8b), INTENT(IN) :: xin, yin, ut1utc
  REAL(KIND=r8b), INTENT(IN) :: dlod, dX, DY

  INTEGER(KIND=i4b) :: ixout, iyout, idpsio, idepso
  INTEGER(KIND=i4b) ::  J, iopt
  INTEGER(KIND=i4b) ::  iyy, imm, idd

  REAL(KIND=r8b) :: dpsi, deps
  REAL(KIND=r8b) :: xout, yout, dmjd, dpsi_out, deps_out
  REAL(KIND=r8b) :: a1utc, a1ut1, dut1, a1utc_rate
  REAL(KIND=r8b) :: JD

  CHARACTER(2) :: hh='00'

  ! old scale was D4
  xout = xin*ScaleD6
  yout = yin*ScaleD6
   
  ixout = xout
  iyout = yout
  
  CALL Calculate_a1ut1( mjd, ut1utc, a1ut1 )

  ! set conversion flag to go from dX,dY to dpsi,deps
  iopt = 2
  dmjd = REAL(mjd,r8b)
  call dXdY_dpsideps(dmjd,dX,dY,dpsi,deps,iopt)
  dpsi_out = dpsi*ScaleD6
  deps_out = deps*ScaleD6

  idpsio = INT(dpsi_out)
  idepso = INT(deps_out)
      
  seq_no = seq_no+1

  DO J = 1,2
    IF(yy(J:J) == ' ') yy(J:J) = '0'
    IF(mm(J:J) == ' ') mm(J:J) = '0'
    IF(dd(J:J) == ' ') dd(J:J) = '0'
  END DO

  !WRITE(UnitNum, "(4A2,2X,2I5,D15.6,4X,2I6,21X,I8)")  &  old format
  WRITE(UnitNum, "(4A2,2(1X,I7),F15.10,4X,2I7,15X,I8)")  &
                   yy,mm,dd,hh,ixout,iyout,a1ut1,idepso,idpsio,seq_no

END SUBROUTINE Output_Pole_Predicts


!****************************************************************************
  SUBROUTINE Lines_Flux_Transfer( year, ymd_flux, Num_Records_Flux,     &
             nlines_flux_transfer, nlines_from_end, ymd_fluxap,         &
             Num_Records_FluxAp, nlines_fluxap_transfer,                &
             nlines_from_end_ap, ymd_fluxkp8, Num_Records_FluxKp8,      &
             nlines_fluxkp8_transfer, nlines_from_end_kp8,              &
             curr_ymd_kp8_beg )
                             
  INTEGER(KIND=i4b), INTENT(IN) :: year, curr_ymd_kp8_beg
  INTEGER(KIND=i4b), INTENT(IN) :: ymd_flux, ymd_fluxap, ymd_fluxkp8
  INTEGER(KIND=i4b), INTENT(IN) :: Num_Records_Flux, Num_Records_FluxAp
  INTEGER(KIND=i4b), INTENT(IN) :: Num_Records_FluxKp8
  INTEGER(KIND=i4b), INTENT(INOUT) :: nlines_flux_transfer
  INTEGER(KIND=i4b), INTENT(INOUT) :: nlines_fluxap_transfer
  INTEGER(KIND=i4b), INTENT(INOUT) :: nlines_fluxkp8_transfer
  INTEGER(KIND=i4b), INTENT(INOUT) :: nlines_from_end, nlines_from_end_ap
  INTEGER(KIND=i4b), INTENT(INOUT) :: nlines_from_end_kp8

  INTEGER(KIND=i4b) :: yy, mm_flux, mm_fluxap, mm_fluxkp8, diff_yy, yy_last
  INTEGER(KIND=i4b) :: mjd_last_fluxkp8, mjd_prev_fluxkp8, mjd_fluxkp8
  INTEGER(KIND=i4b) :: mjd_fluxkp8_start

  ! compute number of lines to transfer for flux
  yy = ymd_flux/10000
  mm_flux = (ymd_flux - yy*10000)/100
  IF( yy < 50 ) THEN
     yy = yy + 2000
  ELSE
     yy = yy + 1900
  END IF
  nlines_from_end = 0
  diff_yy = yy - year
  write(*,*) 'diff_yy ',diff_yy
  IF( (diff_yy ) > 0 ) THEN
    nlines_from_end = diff_yy * 12 * nlines_per_month
    write(*,*) 'nlines_from_end ',nlines_from_end
  END IF

  nlines_from_end = nlines_from_end + (mm_flux * nlines_per_month)
  write(*,*) "ymd_flux mm_flux ", ymd_flux, mm_flux
  write(*,*) 'nlines_from_end ',nlines_from_end
  nlines_flux_transfer = Num_Records_Flux - nlines_from_end
  write(*,*) 'flux ',nlines_flux_transfer, nlines_from_end
       
  yy = ymd_fluxap/10000
  mm_fluxap = (ymd_fluxap - yy*10000)/100
  IF( yy < 50 ) THEN
     yy = yy + 2000
  ELSE
     yy = yy + 1900
  END IF
  nlines_from_end_ap = 0
  diff_yy = yy - year
  IF( (diff_yy ) > 0 ) THEN
    nlines_from_end_ap = diff_yy * 12 * nlines_per_month
  END IF

  nlines_from_end_ap = nlines_from_end_ap + (mm_fluxap * nlines_per_month)
  nlines_fluxap_transfer = Num_Records_FluxAP - nlines_from_end_ap

  write(*,*) 'fluxap ', nlines_fluxap_transfer, nlines_from_end_ap

  CALL YMD_MJD( ymd_fluxkp8, mjd_fluxkp8)
  CALL YMD_MJD( curr_ymd_kp8_beg, mjd_prev_fluxkp8)
  CALL YMD_MJD( ymd_fluxkp8_start, mjd_fluxkp8_start)
  write(*,*) "ymd_fluxkp8, curr_ymd_kp8_beg ", ymd_fluxkp8, curr_ymd_kp8_beg
  write(*,*) "mjd_fluxkp8, mjd_prev_fluxkp8 ", mjd_fluxkp8, mjd_prev_fluxkp8
  write(*,*) 'curr_ymd_kp8_beg ymd_fluxkp8 ',curr_ymd_kp8_beg, ymd_fluxkp8
  nlines_fluxkp8_transfer = mjd_prev_fluxkp8 - mjd_fluxkp8_start

  nlines_from_end_kp8 = (mjd_fluxkp8 - mjd_prev_fluxkp8)
  !nlines_fluxkp8_transfer = (Num_Records_FluxKP8 - nlines_from_end_kp8) -1
  write(*,*) 'fluxkp8 ', nlines_fluxkp8_transfer, nlines_from_end_kp8

  END SUBROUTINE Lines_Flux_Transfer



!****************************************************************************
  SUBROUTINE POLE( pole_section )

  CHARACTER(7), INTENT(IN) :: pole_section

  CHARACTER(80) :: Aline

  INTEGER(KIND=i4b) :: Num_Records, UnitNum_Out
  INTEGER(KIND=i4b) :: seq_no

  LOGICAL :: LUPDATE_Curr, LUPDATE_Pred, LWRITE


  !write(*,*) 'pole_section ',pole_section

  INQUIRE( FILE = pole_curr_file, EXIST = LUPDATE_Curr )
  INQUIRE( FILE = pole_pred_file, EXIST = LUPDATE_Pred )


  WRITE(Report_UnitNum,  &
    "('********************************************************************************')")
  WRITE(Report_UnitNum, "(A/)")  pole_section
  WRITE(Report_UnitNum, "(/' Current Pole Values:')")

  IF( LUPDATE_Curr ) THEN
     WRITE(Report_UnitNum, *) "   The file, ",pole_curr_file, ", contains x and y of the pole, UT1-UTC"
     WRITE(Report_UnitNum, *) "   and the celestial pole offsets."
     WRITE(Report_UnitNum, *) 
  ELSE
     WRITE(Report_UnitNum, *) 
     WRITE(Report_UnitNum, *) "   ---------------------------------------------------------------------------"
     WRITE(Report_UnitNum, *) "   The file, ", pole_curr_file, ", does not exist in the current directory." 
     WRITE(Report_UnitNum, *) "   The ", TRIM(pole_section), " section will not be updated in the &
       &master bih table."
     WRITE(Report_UnitNum, *) "   ---------------------------------------------------------------------------"
  END IF

  IF( .NOT. LUPDATE_Curr .AND. .NOT. LUPDATE_Pred ) THEN
     LWRITE = .TRUE.
     UnitNum_Out = NEWBIH_UnitNum
  ELSE
    ! SCRATCH FILE
    OPEN(UNIT = Scratch_UnitNum, FORM = "FORMATTED", STATUS = "SCRATCH", &
       IOSTAT = OpenStatus )
    IF (OpenStatus > 0) THEN
       WRITE( Error_UnitNum,* ) "ERROR *** Cannot open scratch file."
       STOP
    END IF

    ! open second scratch file to write updates
    OPEN(UNIT = Scratch2_UnitNum, FORM = "FORMATTED", STATUS = "SCRATCH", &
       IOSTAT = OpenStatus )
    IF (OpenStatus > 0) THEN
       WRITE( Error_UnitNum,* ) "ERROR *** Cannot open scratch file."
       STOP
    END IF

    LWRITE = .FALSE.
    UnitNum_Out = Scratch_UnitNum
  END IF

  CALL Section_Header( pole_section, Num_Records, LWRITE )
  CALL Transfer_Master_Section( OLDBIH_UnitNum, UnitNum_Out, Num_Records )

  ! update entire pole series if file exists
  IF( LUPDATE_Curr ) THEN
    UnitNum_Out = Scratch2_UnitNum
    CALL Update_Pole( Scratch_UnitNum, UnitNum_Out, seq_no )
  ELSE
    seq_no = Num_Records
    UnitNum_Out = Scratch_UnitNum
  END IF
     
  WRITE(Report_UnitNum, "(//' Predicted Pole Values:')")

  ! update POLE section with predictions if file exists
  IF( LUPDATE_Pred ) THEN
    WRITE(Report_UnitNum, *) "   The file, ",pole_pred_file,", contains predictions for x and y &
         &of the pole, UT1-UTC, dX and dY."
    CALL Pole_Pred( UnitNum_Out, seq_no )
  ELSE
    WRITE(Report_UnitNum, *) 
    WRITE(Report_UnitNum, *) "   -------------------------------------------------------------"
    WRITE(Report_UnitNum, *) "   The file, ", pole_pred_file, ", does not exist in the current directory." 
    WRITE(Report_UnitNum, *) "   No ",TRIM(pole_section)," predictions were written in the master bih table."
    WRITE(Report_UnitNum, *) "   -------------------------------------------------------------"
  END IF 
     
  IF( LUPDATE_Curr .OR. LUPDATE_Pred ) THEN 

    ! write new section header with updated num of records
     CALL Update_Header( pole_section, seq_no )
     REWIND( UnitNum_Out )
   
    ! write updated scratch file to new bih table
    CALL Transfer_Master_Section( UnitNum_Out, NEWBIH_UnitNum, seq_no )

    CLOSE( Scratch_UnitNum )
    CLOSE( Scratch2_UnitNum )
  END IF
 
  WRITE(Report_UnitNum,  &
    "(/'********************************************************************************'//)")

  END SUBROUTINE POLE



!****************************************************************************
  SUBROUTINE Pole_Pred( UnitNum, seq_no )

  INTEGER(KIND=i4b), INTENT(IN) :: UnitNum
  INTEGER(KIND=i4b), INTENT(INOUT) :: seq_no

  CHARACTER(190) :: Aline
  CHARACTER(24) :: File_Date
  CHARACTER(2) ::  yy, mm, dd, hh
  CHARACTER(4) :: ccyy

  INTEGER(KIND=i4b) :: ymd_last, mjd_first, mjd_last, mjd, cymd
  INTEGER(KIND=i4b) :: Num_Records, J
  INTEGER(KIND=i4b) :: iymd

  REAL(KIND=r8b) :: xin, yin, ut1utc
  REAL(KIND=r8b) :: dmjd, dmjd_first
  REAL(KIND=r8b) :: dlod, dX, dY

  LOGICAL :: lfound, lcurrent, lrapid_used

  ! Open pole file
  OPEN(UNIT = POLE_Pred_UnitNum, FILE = pole_pred_file, FORM = "FORMATTED",&
      STATUS = "OLD", IOSTAT = OpenStatus )
  IF (OpenStatus > 0) THEN
    WRITE( Error_UnitNum,* ) "ERROR *** Cannot open input file ", pole_pred_file
    STOP 16
  END IF

  ! Get date of last update
  BACKSPACE(UnitNum)
  READ(UnitNum,"(I6)") ymd_last
  IF( ymd_last/10000 < 50 ) THEN
     ymd_last = ymd_last + 20000000
  ELSE 
     ymd_last = ymd_last + 19000000
  END IF

  CALL YMD_MJD(ymd_last, mjd_last)

  ! read file header
  lfound = .FALSE.
  lcurrent = .FALSE.
  lrapid_used = .FALSE.

  ! Position file to point to date of prediction
  PolePred_Hdr: DO
     READ( POLE_Pred_UnitNum, "(A)", IOSTAT = InputStatus ) Aline
     IF( InputStatus < 0 ) THEN
       EXIT PolePred_Hdr
     END IF

     READ(Aline,"(3(A2),1X,F8.2)") yy, mm, dd, dmjd

           mjd = INt(dmjd)

           IF( mjd <= mjd_last ) THEN
              CYCLE
           END IF
           
           CALL MJD_YMD( mjd, iymd )
           !write(*,*) iymd, dmjd
          
           lfound = .TRUE.
           mjd_last = mjd

           IF( lfound ) THEN
             EXIT PolePred_Hdr
           ELSE 
             WRITE(Error_UnitNum,*) 
             WRITE(Error_UnitNum,*) "ERROR: A gap exists between the Pole Values and the Pole Prediction Values" 
             WRITE(Error_UnitNum,*) "  The date of the last Pole Value is ",ymd_last
             WRITE(Error_UnitNum,*) "  The date of the first Pole Prediction Values is ", iymd
             WRITE(Error_UnitNum, "(/' No Pole Prediction values will be written'/)")  
             RETURN
           END IF 

  END DO PolePred_Hdr

  DO J = 1,2
    IF(mm(J:J) == ' ') mm(J:J) = '0'
    IF(dd(J:J) == ' ') dd(J:J) = '0'
  END DO
  WRITE(Report_UnitNum, "( /'   ---------------------------------------')")
  WRITE(Report_UnitNum, "(  '   Predictions from ',I8)",ADVANCE="NO") &
      iymd


  BACKSPACE(POLE_Pred_UnitNum)

  ! Read in predictions
  ReadPred: DO
     READ( POLE_Pred_UnitNum, "(A)", IOSTAT = InputStatus ) Aline
     IF (InputStatus < 0) THEN
       EXIT ReadPred
     END IF

     READ(Aline,"(3(A2),1X,F8.2,3X,F9.6,10X,F9.6,12X,F10.7,11X,F7.4,11X,F9.3,10X,F9.3)") &
              yy, mm, dd, dmjd, xin, yin, ut1utc, dlod, dX, dY

     mjd = INT(dmjd)

     IF( mjd < mjd_last ) THEN
        CYCLE
     END IF

     ! dX, dy values are in msec of arc; Scale to match units in eopc04_IAU2000.62-now
     dX = dX * ScaleM3
     dY = dY * ScaleM3

     !write(*,*) yy, mm, dd, dmjd, xin, yin, ut1utc, dlod, dX, dY

     CALL Output_Pole_Predicts( UnitNum, yy, mm, dd, mjd, xin, yin, &
                                ut1utc, dlod, dX, dY, seq_no )


  END DO ReadPred

  CALL MJD_YMD( mjd, iymd )

  WRITE(Report_UnitNum, "(' to ',I8)") iymd
  WRITE(Report_UnitNum, "('   ---------------------------------------')")


  END SUBROUTINE Pole_Pred


!****************************************************************************
  SUBROUTINE Section_Header( section, Num_Records, LWRITE )

  CHARACTER(7), INTENT(IN) :: section

  INTEGER(KIND=i4b), INTENT(INOUT) :: Num_Records

  LOGICAL, INTENT(IN) :: LWRITE

  INTEGER(KIND=i4b) ::  I

  !write(*,*) "section to read ", section

  READ( OLDBIH_UnitNum, "(A)", IOSTAT = InputStatus ) Aline
  IF (InputStatus /= 0) THEN
    WRITE( Error_UnitNum, * ) "ERROR *** Input file error reading ", old_bih_file
    STOP 16
  END IF
  IF( Aline(1:7) /= section ) THEN 
    WRITE( Error_UnitNum, * ) "ERROR *** Problem with format of ", old_bih_file
    WRITE( Error_UnitNum, * ) "ERROR *** Check file."
    STOP 16
  END IF

  ! get number of records to read
  READ(Aline,"(8X,I7)") Num_Records

  IF(LWRITE) THEN
    WRITE(NEWBIH_UnitNum, "(A)") Aline
  END IF

  END SUBROUTINE Section_Header


!****************************************************************************
  SUBROUTINE SOLAR_FLUX( flux_section, fluxap_section, fluxkp8_section, &
                         today_date )

  CHARACTER(7), INTENT(IN) :: flux_section, fluxap_section, fluxkp8_section
  CHARACTER(10), INTENT(IN) :: today_date

  CHARACTER(80) :: Aline
  CHARACTER(8) :: cymd
  CHARACTER(4) :: ccyy_curr, ccyy_prev, ccyy_today
  CHARACTER(2) :: mm_curr, mm_today

  INTEGER(KIND=i4b) :: Num_Records_Flux, Num_Records_FluxAP, Num_Records_FluxKP8
  INTEGER(KIND=i4b) :: UnitNum_Out
  INTEGER(KIND=i4b) :: seq_no
  INTEGER(KIND=i4b) :: iccyy_prev, iccyy_curr, curr_ymd_kp8_beg
  INTEGER(KIND=i4b) :: ymd_last_flux, ymd_last_fluxap, ymd_last_fluxkp8, mjd
  INTEGER(KIND=i4b) :: mjd_today, mjd_end_month, ymd_end_month, cc, cymd_end_month
  INTEGER(KIND=i4b) :: nlines_flux_transfer, nlines_fluxap_transfer, nlines_fluxkp8_transfer
  INTEGER(KIND=i4b) :: nlines_from_end, nlines_from_end_ap, nlines_from_end_kp8
  INTEGER(KIND=i4b) :: year, idate, cymd_month_begin, ymd_month_begin
  INTEGER(KIND=i4b) :: seq_no_flux, seq_no_fluxap, seq_no_fluxkp8
  INTEGER(KIND=i4b) :: last_date_flux, last_date_fluxap, last_date_fluxkp8
  INTEGER(Kind=i4b), DIMENSION(tot_days) :: ifluxap, ifluxs, idate_flx
  INTEGER(Kind=i4b), DIMENSION(8,tot_days):: iflxkp

  LOGICAL :: LUPDATE_Prev, LUPDATE_Curr, LUPDATE_Est, LWRITE


  ! use today's date to access solar flux files
  ! will update previous year's solar flux values and the
  ! current year's solar flux values up to the the previous month
  ccyy_curr = today_date(1:4)
  mm_curr = today_date(5:6)

  READ(ccyy_curr,"(I4)") iccyy_curr
  iccyy_prev = iccyy_curr - 1
  WRITE(ccyy_prev,"(I4)") iccyy_prev

  ! 3 sets of files are used to update the solar flux, the previous year's values,
  ! the current year's values and the temporary estimated and predicted values 
  ! DSD.txt, DGD.txt, and 45DF.txt. Only check for DSD.txt, the other two files
  ! should also exist. Check will be done later for the other two files, if they
  ! do not exist, an error message will be written.

  INQUIRE( FILE = ccyy_curr, EXIST = LUPDATE_Curr )
  INQUIRE( FILE = ccyy_prev, EXIST = LUPDATE_Prev )
  INQUIRE( FILE = dsd_file, EXIST = LUPDATE_Est )

  WRITE(Report_UnitNum,  &
    "('********************************************************************************')")
  WRITE(Report_UnitNum, "(3(A7)/)")  flux_section, fluxap_section, fluxkp8_section

  AnyUpdate: IF( .NOT. LUPDATE_Prev .AND. .NOT. LUPDATE_Curr .AND. .NOT. LUPDATE_Est ) THEN
     LWRITE = .TRUE.
     UnitNum_Out = NEWBIH_UnitNum

     ! transfer flux values
     CALL Section_Header( flux_section, Num_Records_Flux, LWRITE )
     CALL Transfer_Master_Section( OLDBIH_UnitNum, UnitNum_Out, Num_Records_Flux )

     ! transfer fluxap values
     CALL Section_Header( fluxap_section, Num_Records_FluxAP, LWRITE )
     CALL Transfer_Master_Section( OLDBIH_UnitNum, UnitNum_Out, Num_Records_FluxAP )

     ! transfer fluxkp8 values
     CALL Section_Header( fluxkp8_section, Num_Records_FluxKP8, LWRITE )
     CALL Transfer_Master_Section( OLDBIH_UnitNum, UnitNum_Out, Num_Records_FluxKP8 )

  ELSE AnyUpdate
    ! SCRATCH FILE
    OPEN(UNIT = Scratch_UnitNum, FORM = "FORMATTED", STATUS = "SCRATCH", &
       IOSTAT = OpenStatus )
    IF (OpenStatus > 0) THEN
       WRITE( Error_UnitNum,* ) "ERROR *** Cannot open scratch file."
       STOP
    END IF

    ! open second scratch file to write updates for flux
    OPEN(UNIT = Scratch2_UnitNum, FORM = "FORMATTED", STATUS = "SCRATCH", &
       IOSTAT = OpenStatus )
    IF (OpenStatus > 0) THEN
       WRITE( Error_UnitNum,* ) "ERROR *** Cannot open scratch file."
       STOP
    END IF

    ! open third scratch file to write updates for fluxap
    OPEN(UNIT = Scratch3_UnitNum, FORM = "FORMATTED", STATUS = "SCRATCH", &
       IOSTAT = OpenStatus )
    IF (OpenStatus > 0) THEN
       WRITE( Error_UnitNum,* ) "ERROR *** Cannot open scratch file."
       STOP
    END IF

    ! open fourth scratch file to write updates for fluxkp8
    OPEN(UNIT = Scratch4_UnitNum, FORM = "FORMATTED", STATUS = "SCRATCH", &
       IOSTAT = OpenStatus )
    IF (OpenStatus > 0) THEN
       WRITE( Error_UnitNum,* ) "ERROR *** Cannot open scratch file."
       STOP
    END IF
    LWRITE = .FALSE.
    UnitNum_Out = Scratch_UnitNum

    ! transfer flux values
    CALL Section_Header( flux_section, Num_Records_Flux, LWRITE )
    CALL Transfer_Master_Section( OLDBIH_UnitNum, UnitNum_Out, Num_Records_Flux )

    BACKSPACE(UnitNum_Out)
    READ(UnitNum_Out,"(I6)") ymd_last_flux
    last_date_flux = ymd_last_flux
  
    ! transfer fluxap values
    CALL Section_Header( fluxap_section, Num_Records_FluxAP, LWRITE )
    CALL Transfer_Master_Section( OLDBIH_UnitNum, UnitNum_Out, Num_Records_FluxAP )

    BACKSPACE(UnitNum_Out)
    READ(UnitNum_Out,"(I6)") ymd_last_fluxap

    ! transfer fluxkp8 values
    CALL Section_Header( fluxkp8_section, Num_Records_FluxKP8, LWRITE )
    CALL Transfer_Master_Section( OLDBIH_UnitNum, UnitNum_Out, Num_Records_FluxKP8 )

    BACKSPACE(UnitNum_Out)
    READ(UnitNum_Out,"(I6)") ymd_last_fluxkp8

   write(*,*) 'ymd_last_flux, ymd_last_fluxap, ymd_last_fluxkp8 ', &
       ymd_last_flux, ymd_last_fluxap, ymd_last_fluxkp8

    Set_NLines: IF( .NOT. LUPDATE_Prev .AND. .NOT. LUPDATE_Curr ) THEN
       ! at beginning of year and no current file exist, therefore there
       ! is only data for the last 30 days and predictions
       ! only transfer up to today's date - 30 days
       IF( LUPDATE_Est ) THEN
          ! use the month to determine last date that should be transferred,
          ! data will be updated on a month by month basis
          mm_today = today_date(5:6)
          ccyy_today = today_date(1:4)
          cymd = ccyy_today//mm_today//"01"
          READ(cymd,"(I8)") cymd_month_begin
          write(*,*) "cymd_month_begin ", cymd_month_begin
          CALL YMD_MJD( cymd_month_begin, mjd)
          mjd_end_month = mjd - 1
          CALL MJD_YMD( mjd_end_month, cymd_end_month )
          cc = cymd_end_month/1000000
          ymd_end_month = cymd_end_month - cc*1000000
          cc = cymd_month_begin/1000000
          ymd_month_begin = cymd_month_begin - cc*1000000
          write(*,*) "ymd_end_month ", ymd_end_month
          year = iccyy_curr
          curr_ymd_kp8_beg = ymd_month_begin
          CALL Lines_Flux_Transfer( year, ymd_month_begin, Num_Records_Flux, &
             nlines_flux_transfer, nlines_from_end, ymd_month_begin,         &
             Num_Records_FluxAp, nlines_fluxap_transfer,                     &
             nlines_from_end_ap, ymd_last_fluxkp8, Num_Records_FluxKp8,      &
             nlines_fluxkp8_transfer, nlines_from_end_kp8, curr_ymd_kp8_beg )
          last_date_flux = ymd_end_month
       ELSE
           nlines_flux_transfer = Num_Records_Flux 
           nlines_fluxap_transfer = Num_Records_FluxAP 
           nlines_fluxkp8_transfer = Num_Records_FluxKP8 
       END IF
    ELSE Set_NLines
       IF( LUPDATE_Prev ) THEN
          year = iccyy_prev
       ELSE
          year = iccyy_curr
       END IF
       ! date of Jan 1, of the current year
       curr_ymd_kp8_beg = year*10000 + 100 + 1
       CALL Lines_Flux_Transfer( year, ymd_last_flux, Num_Records_Flux,   &
          nlines_flux_transfer, nlines_from_end, ymd_last_fluxap,         &
          Num_Records_FluxAp, nlines_fluxap_transfer,                     &
          nlines_from_end_ap, ymd_last_fluxkp8, Num_Records_FluxKp8,      &
          nlines_fluxkp8_transfer, nlines_from_end_kp8, curr_ymd_kp8_beg )

    END IF Set_NLines

    REWIND(UnitNum_Out)
    ! transfer flux values preparing to add new information
    CALL Transfer_Master_Section( UnitNum_Out, Scratch2_UnitNum, nlines_flux_transfer )
    IF( nlines_flux_transfer .NE. Num_Records_Flux ) THEN
       ! set out unit to -1 to skip lines and not write
       ! stores last 4 months data values
       CALL Transfer_Master_Section ( UnitNum_Out, -1, nlines_from_end, FLX = ifluxs, &
                                      DATE_ARRAY = idate_flx )
    END IF

    ! transfer fluxap values preparing to add new information
    CALL Transfer_Master_Section( UnitNum_Out, Scratch3_UnitNum, nlines_fluxap_transfer )
    IF( nlines_fluxap_transfer .NE. Num_Records_FluxAP ) THEN
       ! set out unit to -1 to skip lines and not write
       ! stores last 4 months data values
       CALL Transfer_Master_Section( UnitNum_Out, -1, nlines_from_end_ap, FLXAP = ifluxap, &
                                      DATE_ARRAY = idate_flx )
    END IF

    ! transfer fluxkp8 values preparing to add new information
    CALL Transfer_Master_Section( UnitNum_Out, Scratch4_UnitNum, nlines_fluxkp8_transfer )
    IF( nlines_fluxkp8_transfer .NE. Num_Records_FluxKP8 ) THEN
       ! set out unit to -1 to skip lines and not write
       ! stores last 4 months data values
       CALL Transfer_Master_Section( UnitNum_Out, -1, nlines_from_end_kp8, FLXKP = iflxkp, &
                                      DATE_ARRAY = idate_flx )
    END IF

  END IF AnyUpdate

  seq_no_flux = nlines_flux_transfer
  seq_no_fluxap = nlines_fluxap_transfer
  seq_no_fluxkp8 = nlines_fluxkp8_transfer

  ! update solar flux values if previous file  
  WRITE(Report_UnitNum, *) 
  WRITE(Report_UnitNum, "(' Previous ',3(A7),' values:')")  flux_section, fluxap_section, fluxkp8_section
  Update_Prev: IF( LUPDATE_Prev ) THEN
    WRITE(Report_UnitNum, *) "   The file, ",ccyy_prev, ", contains the previous year's history of"
    WRITE(Report_UnitNum, *) "   KP8, KP, AP and SFLUX F10.7 values. This file is sometimes "
    WRITE(Report_UnitNum, *) "   updated well into the current year."
    OPEN(UNIT = FLUX_Prev_UnitNum, FILE = ccyy_prev, FORM = "FORMATTED",&
        STATUS = "OLD", IOSTAT = OpenStatus )
    IF (OpenStatus > 0) THEN
        WRITE( Error_UnitNum,* ) "ERROR *** Cannot open input file ", ccyy_prev
        STOP 16
    END IF

    CALL Update_Solar_Flux( FLUX_Prev_UnitNum, Scratch2_UnitNum, Scratch3_UnitNum,        &
                            Scratch4_UnitNum, seq_no_flux, seq_no_fluxap, seq_no_fluxkp8, &
                            last_date_flux )
    !write(*,*) 'seq_no_flux seq_no_fluxap seq_no_fluxkp8 ',seq_no_flux, seq_no_fluxap, &
    !  seq_no_fluxkp8
  ELSE Update_Prev
     WRITE(Report_UnitNum, *) 
     WRITE(Report_UnitNum, *) "   -----------------------------------------------------------------"
     WRITE(Report_UnitNum, *) "   The file, ", ccyy_prev, ", does not exist in the current directory." 
     WRITE(Report_UnitNum, *) "   The ", TRIM(flux_section),", ",TRIM(fluxap_section),", and ", &
       &TRIM(fluxkp8_section), " section historical values"
     WRITE(Report_UnitNum, *) "   will not be updated in the master bih table."
     WRITE(Report_UnitNum, *) "   -----------------------------------------------------------------"
     WRITE(Report_UnitNum, *) 
  END IF Update_Prev

  ! update solar flux values if current file  
  WRITE(Report_UnitNum, "(/' Current ',3(A7),' values:')")  flux_section, fluxap_section, fluxkp8_section
  Update_Curr: IF( LUPDATE_Curr ) THEN
    WRITE(Report_UnitNum, *) "   The file, ",ccyy_curr, ", contains the current year's values of"
    WRITE(Report_UnitNum, *) "   KP8, KP, AP and SFLUX F10.7 values up to the end of the"
    WRITE(Report_UnitNum, *) "   previous month."

    OPEN(UNIT = FLUX_Curr_UnitNum, FILE = ccyy_curr, FORM = "FORMATTED",&
        STATUS = "OLD", IOSTAT = OpenStatus )
    IF (OpenStatus > 0) THEN
      WRITE( Error_UnitNum,* ) "ERROR *** Cannot open input file ", ccyy_curr
      STOP 16
    END IF

    CALL Update_Solar_Flux( FLUX_Curr_UnitNum, Scratch2_UnitNum, Scratch3_UnitNum,        &
                            Scratch4_UnitNum, seq_no_flux, seq_no_fluxap, seq_no_fluxkp8, &
                            last_date_flux )
    !write(*,*) 'seq_no_flux seq_no_fluxap seq_no_fluxkp8 ',seq_no_flux, seq_no_fluxap,&
    !   seq_no_fluxkp8
  ELSE Update_Curr
     WRITE(Report_UnitNum, *) 
     WRITE(Report_UnitNum, *) "   -----------------------------------------------------------------"
     WRITE(Report_UnitNum, *) "   The file, ", ccyy_curr, ", does not exist in the current directory." 
     WRITE(Report_UnitNum, *) "   The ", TRIM(flux_section),", ",TRIM(fluxap_section),", and ", &
       &TRIM(fluxkp8_section), " section current values"
     WRITE(Report_UnitNum, *) "   will not be updated in the master bih table."
     WRITE(Report_UnitNum, *) "   -----------------------------------------------------------------"
     WRITE(Report_UnitNum, *) 
  END IF Update_Curr

  ! add last 30 days and predicted FLUXS, FLUXAP and FLUXKP8 values
  CALL Solar_Flux_Pred( Scratch2_UnitNum, Scratch3_UnitNum, Scratch4_UnitNum,       &
                       seq_no_flux, seq_no_fluxap, seq_no_fluxkp8, last_date_flux, &
                       flux_section, fluxap_section, fluxkp8_section, ifluxs,     &
                       ifluxap, iflxkp, idate_flx, LUPDATE_Est )

  IF( LUPDATE_Prev .OR. LUPDATE_Curr .OR. LUPDATE_Est ) THEN
     ! write new flux section header with updated num of records
     CALL Update_Header( flux_section, seq_no_flux )
     REWIND( Scratch2_UnitNum )
   
     ! write updated scratch file to new bih table
     CALL Transfer_Master_Section( Scratch2_UnitNum, NEWBIH_UnitNum, seq_no_flux )

     ! write new fluxap section header with updated num of records
     CALL Update_Header( fluxap_section, seq_no_fluxap )
     REWIND( Scratch3_UnitNum )
   
     ! write updated scratch file to new bih table
     CALL Transfer_Master_Section( Scratch3_UnitNum, NEWBIH_UnitNum, seq_no_fluxap )

     ! write new fluxkp8 section header with updated num of records
     CALL Update_Header( fluxkp8_section, seq_no_fluxkp8 )
     REWIND( Scratch4_UnitNum )
   
     ! write updated scratch file to new bih table
     CALL Transfer_Master_Section( Scratch4_UnitNum, NEWBIH_UnitNum, seq_no_fluxkp8 )

     CLOSE( Scratch_UnitNum )
     CLOSE( Scratch2_UnitNum )
     CLOSE( Scratch3_UnitNum )
     CLOSE( Scratch4_UnitNum )

  END IF

  WRITE(Report_UnitNum,  &
    "('********************************************************************************'//)")

  END SUBROUTINE SOLAR_FLUX


!****************************************************************************
  SUBROUTINE Solar_Flux_Pred( UnitNum_Flux, UnitNum_FluxAP, UnitNum_FluxKP8, &
                              seq_no_flux, seq_no_fluxap, seq_no_fluxkp8,    &
                              last_date_flux, flux_section, fluxap_section,  &
                              fluxkp8_section, ifluxs_old, ifluxap_old,      &
                              iflxkp_old, idate_flx, LUPDATE_Est )

  INTEGER, INTENT(IN) :: UnitNum_Flux, UnitNum_FluxAP, UnitNum_FluxKP8
  INTEGER, INTENT(INOUT) :: seq_no_flux, seq_no_fluxap, seq_no_fluxkp8
  INTEGER, INTENT(INOUT) :: last_date_flux
  INTEGER(Kind=i4b), INTENT(INOUT), DIMENSION(:) :: ifluxap_old, ifluxs_old
  INTEGER(Kind=i4b), INTENT(INOUT), DIMENSION(:,:):: iflxkp_old
  INTEGER(Kind=i4b), DIMENSION(:):: idate_flx

  CHARACTER(7), INTENT(IN) :: flux_section, fluxap_section, fluxkp8_section

  LOGICAL, INTENT(IN) :: LUPDATE_Est

  INTEGER, PARAMETER :: number_lines = 3        ! 3 lines of 12 values
  INTEGER, PARAMETER :: days_for_month = 36     ! 36 days for storage for each month
  INTEGER, PARAMETER :: values_per_line = 12    ! 12 values per line
  INTEGER, PARAMETER :: kp8_incrmt = 33         ! increment of kp8 values
  INTEGER, PARAMETER :: pred_per_line = 5       ! prediction values per line

  CHARACTER(80) :: Aline
  CHARACTER(33) :: File_Date
  CHARACTER(20) :: FRMT
  CHARACTER(7) :: cdate_recrd_tmp
  CHARACTER(7), DIMENSION(pred_per_line) :: cdate
  CHARACTER(6) :: cdate_recrd
  CHARACTER(3) :: month_name
  CHARACTER(1), DIMENSION(8):: cnckpv
   
  INTEGER(Kind=i4b) :: fluxs_recrd
  INTEGER(Kind=i4b) :: ccyy, yy, mm, dd, mm_last, yy_last
  INTEGER(Kind=i4b) :: mm_beg, mm_end, mm_est, mmdd, mm_last_date, mm_curr
  INTEGER(Kind=i4b) :: array_ptr, month_ptr, nmonths
  INTEGER(Kind=i4b) :: idate_recrd, idate_recrd_prev, idate_recrd_tmp
  INTEGER(Kind=i4b), DIMENSION(8):: iflxkp_recrd = 0
  INTEGER(Kind=i4b), DIMENSION(pred_per_line) :: ivalues
  INTEGER(Kind=i4b) :: iapsum, j, I, k
  INTEGER(Kind=i4b) :: prev_diff, curr_diff, iptr, idate_1, idate_2
  INTEGER(Kind=i4b) :: mjd_start, mjd_end, mjd_1, mjd_2, mjd_flx
  INTEGER(Kind=i4b) :: istart, IL1, IL2, yy_flx, mm_flx, dd_flx
  INTEGER(Kind=i4b) :: ymd_begin_curr, ymd_end_curr
  INTEGER(Kind=i4b) :: ymd_begin_pred, ymd_end_pred
  INTEGER(Kind=i4b) :: ymd_flx, idate_value
  INTEGER(Kind=i4b), DIMENSION(tot_days) :: idate
  INTEGER(Kind=i4b), DIMENSION(tot_days) :: ifluxap, ifluxs
  INTEGER(Kind=i4b), DIMENSION(8,tot_days) :: iflxkp

  REAL(KIND=r8b) :: prev_ap_value, curr_ap_value
     
  LOGICAL :: LFIRST, LEXIST_DGD, LEXIST_DF45, LAP, LF10PT7
  LOGICAL :: LFOUND, LFLX, LMONTH


  ! initialize variables
  ifluxs = 0
  ifluxap = 0
  iflxkp = 0
  idate = 0

  yy_last = last_date_flux/10000
  mm_last = (last_date_flux - yy_last*10000)/100
  CALL YMD_MJD( last_date_flux, mjd_2)

  WRITE(Report_UnitNum, "(/' Last 30 Days of  ',3(A7),' values:')")  flux_section, fluxap_section, fluxkp8_section
  IF( LUPDATE_Est ) THEN
     WRITE(Report_UnitNum, *) "   The file, ",dsd_file, ", contains the last 30 days of daily solar data."
  ELSE
     WRITE(Report_UnitNum, *) 
     WRITE(Report_UnitNum, *) "   ---------------------------------------------------------------------------"
     WRITE(Report_UnitNum,*) "   The file, ",dsd_file,", does not exist in the current directory."
     WRITE(Report_UnitNum,*) "   No Solar Flux values from the last 30 days or predicted values were "
     WRITE(Report_UnitNum,*) "   written to the master bih table."
     WRITE(Report_UnitNum, *) "   ---------------------------------------------------------------------------"
     WRITE(Report_UnitNum,*) 
     RETURN
  END IF

  INQUIRE( FILE = dgd_file, EXIST = LEXIST_DGD )
  IF( LEXIST_DGD ) THEN
     WRITE(Report_UnitNum, *) "   The file, ",dgd_file, ", contains the last 30 days of daily geomagnetic data."
  ELSE
     WRITE(Report_UnitNum, *) 
     WRITE(Report_UnitNum, *) "   ---------------------------------------------------------------------------"
     WRITE(Report_UnitNum,*) "   The file, ",dgd_file,", does not exist in the current directory."
     WRITE(Report_UnitNum,*) "   No Solar Flux values from the last 30 days or predicted values were "
     WRITE(Report_UnitNum,*) "   written to the master bih table."
     WRITE(Report_UnitNum, *) "   ---------------------------------------------------------------------------"
     WRITE(Report_UnitNum,*) 
     RETURN
  END IF

  !!!!!!!!!!!!!!!!!!!
  ! Open DSD.txt file
  OPEN(UNIT = DSD_UnitNum, FILE = dsd_file, FORM = "FORMATTED",&
      STATUS = "OLD", IOSTAT = OpenStatus )
  IF (OpenStatus > 0) THEN
    WRITE( Error_UnitNum,* ) "ERROR *** Cannot open input file ", dsd_file
    STOP 16
  END IF

  ! skip header 
  DSD_Hdr: DO
    READ( DSD_UnitNum, "(A)", IOSTAT = InputStatus ) Aline
    IF (InputStatus /= 0) THEN
      WRITE( Error_UnitNum, * ) "ERROR *** Input file error reading input file ", &
          dsd_file
      STOP 16
    END IF

    IF(INDEX(Aline,"Issued") > 0) THEN
       File_Date = Aline(8:40)
       WRITE(Report_UnitNum,*) "   ",dsd_file," File Date: ", File_Date
    END IF

    IF( Aline(4:7) .EQ. "Date" ) THEN
       EXIT DSD_Hdr
    END IF
  END DO DSD_Hdr

  !!!!!!!!!!!!!!!!!!!
  ! Open DGD.txt file
  OPEN(UNIT = DGD_UnitNum, FILE = dgd_file, FORM = "FORMATTED",&
      STATUS = "OLD", IOSTAT = OpenStatus )
  IF (OpenStatus > 0) THEN
    WRITE( Error_UnitNum,* ) "ERROR *** Cannot open input file ", dgd_file
    STOP 16
  END IF

  ! skip header
  DGD_Hdr: DO
    READ( DGD_UnitNum, "(A)", IOSTAT = InputStatus ) Aline
    IF (InputStatus /= 0) THEN
      WRITE( Error_UnitNum, * ) "ERROR *** Input file error reading input file ", &
          dgd_file
      STOP 16
    END IF

    IF(INDEX(Aline,"Issued") > 0) THEN
       File_Date = Aline(8:40)
       WRITE(Report_UnitNum,*) "   ",dgd_file," File Date: ", File_Date
    END IF

    IF( Aline(4:7) .EQ. "Date" ) THEN
       EXIT DGD_Hdr
    END IF
  END DO DGD_Hdr

  ! Read DSD.txt file
  LFIRST = .FALSE.
  DSD_Data: DO
    READ( DSD_UnitNum, "(A)", IOSTAT = InputStatus ) Aline
    IF( InputStatus < 0 ) THEN
      EXIT DSD_Data
    END IF
    IF (InputStatus /= 0) THEN
      WRITE( Error_UnitNum, * ) "ERROR *** Input file error reading input file ", &
          dsd_file
      STOP 16
    END IF
    IF( ALINE(1:1) .EQ. "#" ) THEN 
        CYCLE
    END IF
    READ(ALINE,"(I4,1X,I2,1X,I2,2X,I3)" ) ccyy, mm, dd, fluxs_recrd 
    !write(*,*) ccyy, mm, dd, fluxs_recrd 
    yy = mod(ccyy,100)
    
    idate_recrd = yy * 10000 + mm * 100 + dd
    !write(*,*) "idate_recrd ", idate_recrd
        
    IF( .NOT. LFIRST ) THEN
       mm_beg = mm

       ! compare date of DSD.txt file with last flux date
       ! if there exist a gap, fill in with previously stored
       ! current values
       idate_1 = ccyy * 10000 + mm * 100 + dd
       !write(*,*) "idate_1, last_date_flux ", idate_1, last_date_flux
       CALL YMD_MJD( idate_1, mjd_1 )
       !write(*,*) 'mjd_1 mjd_2 ',mjd_1, mjd_2
       IF(mjd_1 - mjd_2 > 1) THEN
         mjd_start = mjd_2 + 1
         mjd_end = mjd_1 - 1 
         !write(*,*) 'mjd_start mjd_end ', mjd_start, mjd_end
         istart = 1
         LFLX = .FALSE.
         DO IL1 = mjd_start, mjd_end
            !write(*,*) "IL1 istart ", IL1, istart
            CALL MJD_YMD(IL1,ymd_flx)
           DO IL2 = istart, tot_days
              yy_flx = idate_flx(IL2)/10000
              idate_value = idate_flx(IL2)
              IF( yy_flx < 50 ) THEN
                 idate_value = idate_value + 20000000
              ELSE
                 idate_value = idate_value + 19000000
              END IF
              IF( idate_value == ymd_flx ) THEN
                  !write(*,*) 'idate_flx(IL2) ', idate_flx(IL2)
                  istart = IL2 
                  mm_flx = (idate_flx(IL2) - yy_flx*10000)/100
                  dd_flx = idate_flx(IL2) - yy_flx*10000 - mm_flx*100
                  !write(*,*) 'yy_flx, mm_flx ', yy_flx, mm_flx
                  mm_curr = mm_flx
                  IF( mm_curr < mm_beg ) THEN
                     mm_curr = mm_curr + 12
                  END IF
                  array_ptr = ((mm_curr-mm_beg)*days_for_month) + dd_flx

                  idate(array_ptr) = idate_flx(IL2)
                  ifluxs(array_ptr)= ifluxs_old(IL2)

                  !write(*,*) "array_ptr, idate ifluxs ", &
                  !   array_ptr, idate(array_ptr), ifluxs(array_ptr)
                  LFLX = .TRUE.
                  EXIT
               END IF
            END DO
            ! Data to fill in gap not found, write message
            IF( .NOT. LFLX ) THEN
              WRITE(Report_UnitNum,*) 
              WRITE(Report_UnitNum,*) "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!" 
              WRITE(Report_UnitNum,*) "ERROR: A gap exists between the Flux Values and the Flux Values from DSD.txt file" 
              WRITE(Report_UnitNum,*) "  The date of the last Flux Value is ",idate_2
              WRITE(Report_UnitNum,*) "  The date of the first Flux Value from the last 30 days is ", idate_1
              WRITE(Report_UnitNum, "(/' No Current or Prediction values for ',3(A7),' will be written')")  &
                   flux_section, fluxap_section, fluxkp8_section
              WRITE(Report_UnitNum,*) "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!" 
              WRITE(Report_UnitNum,*) 
              WRITE(Error_UnitNum,*) 
              WRITE(Error_UnitNum,*) "ERROR: A gap exists between the Flux Values and the Flux Values from DSD.txt file" 
              WRITE(Error_UnitNum,*) "  The date of the last Flux Value is ",last_date_flux
              WRITE(Error_UnitNum,*) "  The date of the first Flux Value from the last 30 days is ", idate_1
              WRITE(Error_UnitNum, "(/' No Current or Prediction values for ',3(A7),' will be written'/)")  &
              flux_section, fluxap_section, fluxkp8_section
              RETURN
           END IF
         END DO   
       END IF
       ymd_begin_curr = idate_recrd
       LFIRST = .TRUE.
    ELSE
       mm_end = mm
       IF( mm_beg > mm_end ) THEN
         mm_end = mm_end + 12
       END IF
    END IF
    ymd_end_curr = idate_recrd
    !write(*,*) "ymd_begin_curr ymd_end_curr", ymd_begin_curr, ymd_end_curr
       
    mm_curr = mm
    IF( mm_curr < mm_beg ) THEN
       mm_curr = mm_curr + 12
    END IF
    array_ptr = ((mm_curr-mm_beg)*days_for_month) + dd

    idate(array_ptr) = idate_recrd
    ifluxs(array_ptr)=fluxs_recrd*10
    !write(*,*) 'array_ptr idate ifluxs',array_ptr,idate(array_ptr),ifluxs(array_ptr)

  END DO DSD_Data


  ! Read DGD.txt file
  LFIRST = .FALSE.
  DGD_Data: DO
    READ( DGD_UnitNum, "(A)", IOSTAT = InputStatus ) Aline
    IF( InputStatus < 0 ) THEN
      EXIT DGD_Data
    END IF
    IF (InputStatus /= 0) THEN
      WRITE( Error_UnitNum, * ) "ERROR *** Input file error reading input file ", &
          dgd_file
      STOP 16
    END IF
    IF( ALINE(1:1) .EQ. "#" ) THEN 
        CYCLE
    END IF
    READ(ALINE,"(I4,1X,I2,1X,I2,49X,I3,1X,8(A1,I1))" )  &
        ccyy, mm, dd, iapsum, (cnckpv(j), iflxkp_recrd(j),j=1,8)
    !write(*,*) ccyy, mm, dd, iflxkp_recrd
    yy = mod(ccyy,100)
    
    idate_recrd = yy * 10000 + mm * 100 + dd

    IF( .NOT. LFIRST ) THEN
       mm_beg = mm

       ! compare date of DGD.txt file with last fluxap date
       ! if there exist a gap, fill in with previously stored
       ! current values
       idate_1 = ccyy * 10000 + mm * 100 + dd
       !write(*,*) "idate_1, last_date_flux ", idate_1, last_date_flux
       CALL YMD_MJD( idate_1, mjd_1 )
       !write(*,*) 'mjd_1 mjd_2 ',mjd_1, mjd_2
       IF(mjd_1 - mjd_2 > 1) THEN
         mjd_start = mjd_2 + 1
         mjd_end = mjd_1 - 1 
         !write(*,*) 'mjd_start mjd_end ', mjd_start, mjd_end
         istart = 1
         LFLX = .FALSE.
         DO IL1 = mjd_start, mjd_end
            !write(*,*) "IL1 istart ", IL1, istart
            CALL MJD_YMD(IL1,ymd_flx)
           DO IL2 = istart, tot_days
              yy_flx = idate_flx(IL2)/10000
              idate_value = idate_flx(IL2)
              IF( yy_flx < 50 ) THEN
                 idate_value = idate_value + 20000000
              ELSE
                 idate_value = idate_value + 19000000
              END IF
              IF( idate_value == ymd_flx ) THEN
                  !write(*,*) 'idate_flx(IL2) ', idate_flx(IL2)
                  istart = IL2 
                  mm_flx = (idate_flx(IL2) - yy_flx*10000)/100
                  dd_flx = idate_flx(IL2) - yy_flx*10000 - mm_flx*100
                  !write(*,*) 'yy_flx, mm_flx ', yy_flx, mm_flx
                  mm_curr = mm_flx
                  IF( mm_curr < mm_beg ) THEN
                     mm_curr = mm_curr + 12
                  END IF
                  array_ptr = ((mm_curr-mm_beg)*days_for_month) + dd_flx

                  ifluxap(array_ptr)= ifluxap_old(IL2)
                  iflxkp(:,array_ptr) = iflxkp_old(:,IL2)

                  !write(*,*) "array_ptr, idate ifluxap ", &
                  !   array_ptr, idate(array_ptr), ifluxap(array_ptr)
                  !write(*,*) "iflxkp ", iflxkp(:,array_ptr)
                  LFLX = .TRUE.
                  EXIT
               END IF
            END DO
            ! Data to fill in gap not found, write message
            IF( .NOT. LFLX ) THEN
              WRITE(Report_UnitNum,*) 
              WRITE(Report_UnitNum,*) "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!" 
              WRITE(Report_UnitNum,*) "ERROR: A gap exists between the Flux Values and the Flux Values from DGD.txt file" 
              WRITE(Report_UnitNum,*) "  The date of the last Flux Value is ",idate_2
              WRITE(Report_UnitNum,*) "  The date of the first Flux Value from the last 30 days is ", idate_1
              WRITE(Report_UnitNum, "(/' No Current or Prediction values for ',3(A7),' will be written')")  &
                   flux_section, fluxap_section, fluxkp8_section
              WRITE(Report_UnitNum,*) "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!" 
              WRITE(Report_UnitNum,*) 
              WRITE(Error_UnitNum,*) 
              WRITE(Error_UnitNum,*) "ERROR: A gap exists between the Flux Values and the Flux Values from DGD.txt file" 
              WRITE(Error_UnitNum,*) "  The date of the last Flux Value is ",last_date_flux
              WRITE(Error_UnitNum,*) "  The date of the first Flux Value from the last 30 days is ", idate_1
              WRITE(Error_UnitNum, "(/' No Current or Prediction values for ',3(A7),' will be written'/)")  &
              flux_section, fluxap_section, fluxkp8_section
              RETURN
           END IF
         END DO   
       END IF
       ymd_begin_curr = idate_recrd
       LFIRST = .TRUE.
    END IF
    mm_curr = mm
    IF( mm_curr < mm_beg ) THEN
       mm_curr = mm_curr + 12
    END IF
    array_ptr = ((mm_curr-mm_beg)*days_for_month) + dd

    ifluxap(array_ptr)=iapsum

    DO i=1,8
       iflxkp(i,array_ptr) = iflxkp_recrd(i)
       if(cnckpv(i).eq."-") then
          iflxkp(i,array_ptr)=(iflxkp(i,array_ptr)  * 100) - kp8_incrmt
       else if(cnckpv(i).eq."+") then
          iflxkp(i,array_ptr)=(iflxkp(i,array_ptr)  * 100) + kp8_incrmt
       else 
          iflxkp(i,array_ptr)=(iflxkp(i,array_ptr) * 100)
       endif
    END DO
   
  END DO DGD_Data

  IF( ymd_begin_curr/10000 < 50 ) THEN
      ymd_begin_curr = ymd_begin_curr + 20000000
  ELSE
      ymd_begin_curr = ymd_begin_curr + 19000000
  END IF
  IF( ymd_end_curr/10000 < 50 ) THEN
      ymd_end_curr = ymd_end_curr + 20000000
  ELSE
      ymd_end_curr = ymd_end_curr + 19000000
  END IF
  WRITE(Report_UnitNum,*)
  WRITE(Report_UnitNum,*) "   ----------------------------------------"
  WRITE(Report_UnitNum,*) "   Last 30 days from",ymd_begin_curr, "to",ymd_end_curr
  WRITE(Report_UnitNum,*) "   ----------------------------------------"
  WRITE(Report_UnitNum,*)
    

  !!!!!!!!!!!!!!!!!!!
  ! Open 45DF.txt file
  WRITE(Report_UnitNum, "(/' Predicted ',3(A7),' values:')")  flux_section, fluxap_section, fluxkp8_section
  INQUIRE( FILE = df45_file, EXIST = LEXIST_DF45 )
  IF( LEXIST_DF45) THEN
     WRITE(Report_UnitNum, *) "   The file, ",df45_file, ", contains the 45-Day AP and F10.7cm Flux prediction."
  ELSE
     WRITE(Report_UnitNum, *) 
     WRITE(Report_UnitNum, *) "   ---------------------------------------------------------------------------"
     WRITE(Report_UnitNum,*) "   The file, ",df45_file,", does not exist in the current directory."
     WRITE(Report_UnitNum,*) "   No Solar Flux predictions values were written to the master bih table."
     WRITE(Report_UnitNum, *) "   ---------------------------------------------------------------------------"
     WRITE(Report_UnitNum,*) 
     RETURN
  END IF
  OPEN(UNIT = DF45_UnitNum, FILE = df45_file, FORM = "FORMATTED",&
      STATUS = "OLD", IOSTAT = OpenStatus )
  IF (OpenStatus > 0) THEN
    WRITE( Error_UnitNum,* ) "ERROR *** Cannot open input file ", df45_file
    STOP 16
  END IF

  ! skip header 
  DF45_Hdr: DO
    READ( DF45_UnitNum, "(A)", IOSTAT = InputStatus ) Aline
    IF (InputStatus /= 0) THEN
      WRITE( Error_UnitNum, * ) "ERROR *** Input file error reading input file ", &
          df45_file
      STOP 16
    END IF

    IF(INDEX(Aline,"Issued") > 0) THEN
       File_Date = Aline(8:40)
       WRITE(Report_UnitNum,*) "   ",df45_file," File Date: ", File_Date
    END IF

    IF( Aline(1:4) .EQ. "#---" ) THEN
       EXIT DF45_Hdr
    END IF
  END DO DF45_Hdr

  ! Read 45DF.txt file
  WRITE(FRMT,"('(',I1,'(A7,1X,I3,1X))')") pred_per_line
  LFIRST = .FALSE.
  DF45_Data: DO
    READ( DF45_UnitNum, "(A)", IOSTAT = InputStatus ) Aline
    IF( InputStatus < 0 ) THEN
      EXIT DF45_Data
    END IF
    IF (InputStatus /= 0) THEN
      WRITE( Error_UnitNum, * ) "ERROR *** Input file error reading input file ", &
          df45_file
      STOP 16
    END IF
    IF( ALINE(1:29) == "45-DAY F10.7 CM FLUX FORECAST" ) THEN
       LAP = .FALSE.
       LF10PT7 = .TRUE.
    ELSE IF( ALINE(1:18) == "45-DAY AP FORECAST" ) THEN
       LAP = .TRUE.
       LF10PT7 = .FALSE.
    ELSE IF( ALINE(1:10) == "FORECASTER" ) THEN
       EXIT DF45_Data
    ELSE
       READ(ALINE,FRMT) (cdate(J),ivalues(J),J=1,pred_per_line)
       !write(*,*) cdate, ivalues

       DO I = 1, pred_per_line
         READ(cdate(I),"(I2,A3,I2)") dd, month_name, yy
         LMONTH = .FALSE.
         DO J = 1, 12
           IF( month_name == months_3(J) ) THEN
              mm = J
              LMONTH = .TRUE.
              EXIT
           END IF
         END DO
         IF( .NOT. LMONTH ) THEN
            WRITE(Error_UnitNum,*) "ERROR: PROGRAM DID NOT MATCH THE MONTH IN THE 45DF.txt.  CHECK FOR FORMAT CHANGE."
            STOP 169
         END IF
         idate_recrd = yy * 10000 + mm * 100 + dd
         !write(*,*) "idate_recrd ", idate_recrd
         mm_end = mm
         IF( mm_beg > mm_end ) THEN
           mm_end = mm_end + 12
         END IF

         mm_curr = mm
         IF( mm_curr < mm_beg ) THEN
            mm_curr = mm_curr + 12
         END IF
         array_ptr = ((mm_curr-mm_beg)*days_for_month) + dd

         ! fill in those dates that have not been filled yet
         ! this will be the prediction values because we want
         ! to overwrite the last temporary estimated values
         IF( idate(array_ptr) .EQ. 0 ) THEN
            idate(array_ptr) = idate_recrd
            IF( .NOT. LFIRST ) THEN
               ymd_begin_pred = idate_recrd
               LFIRST = .TRUE.
            END IF
            ymd_end_pred = idate_recrd
         END IF
         IF( LF10PT7 ) THEN
            ifluxs(array_ptr)=ivalues(I)*10
         ELSE 
            ifluxap(array_ptr) = ivalues(I)

            ! use the kp and ap table values to calculate kp prediction
            ! find values closest
            LFOUND = .FALSE.
            prev_ap_value = ap_table_values(1)
            DO J = 2, num_kpap_table_values
              curr_ap_value = ap_table_values(J)
              IF( ifluxap(array_ptr) >= prev_ap_value .AND.   &
                  ifluxap(array_ptr) <= curr_ap_value ) THEN
                 !WRITE(*,*) 'prev value curr ',prev_ap_value, ifluxap(array_ptr), curr_ap_value
                 ! find closest value, then use index of closest value to get kp
                 prev_diff = ifluxap(array_ptr) - prev_ap_value
                 curr_diff = curr_ap_value - ifluxap(array_ptr)
                 IF( prev_diff < curr_diff ) THEN 
                    iptr = J
                 ELSE
                    iptr = J-1
                 END IF 
                 !write(*,*) 'kp value ',kp_table_values(iptr) * 100
                 iflxkp(1:8,array_ptr) = kp_table_values(iptr) * 100 + .05
                 LFOUND = .TRUE.
                 EXIT
              ELSE
                 prev_ap_value = curr_ap_value
              END IF
           END DO
           IF( .NOT. LFOUND ) THEN
              WRITE(Error_UnitNum,*) "ERROR: fluxap value of ",ifluxap(array_ptr)," can not be"
              WRITE(Error_UnitNum,*) " translated to a kp value in the kp and ap tables."
              WRITE(Error_UnitNum,*) " check program logic or values of ap."
           END IF
              
         END IF

       END DO 
    END IF
  END DO DF45_Data

  IF( ymd_begin_pred/10000 < 50 ) THEN
      ymd_begin_pred = ymd_begin_pred + 20000000
  ELSE
      ymd_begin_pred = ymd_begin_pred + 19000000
  END IF
  IF( ymd_end_pred/10000 < 50 ) THEN
      ymd_end_pred = ymd_end_pred + 20000000
  ELSE
      ymd_end_pred = ymd_end_pred + 19000000
  END IF
  WRITE(Report_UnitNum,*)
  WRITE(Report_UnitNum,*) "   ------------------------------------"
  WRITE(Report_UnitNum,*) "   Predicted from",ymd_begin_pred, "to",ymd_end_pred
  WRITE(Report_UnitNum,*) "   ------------------------------------"
  WRITE(Report_UnitNum,*)

  ! write fluxs section
  nmonths = (mm_end - mm_beg) + 1
  !write(*,*) 'nmonths ',nmonths
  DO I = 1, nmonths
    month_ptr = (I-1) * days_for_month
    !write(*,*) 'month_ptr ',month_ptr
    mm_est = (mm_beg - 1) + I
    !write(*,*) 'mm_est ',mm_est
    IF( mm_last == mm_est) THEN
       CYCLE
    END IF
    idate_recrd_prev = idate(month_ptr+1)
    DO J = 1,number_lines
       seq_no_flux = seq_no_flux + 1
       idate_recrd = idate(month_ptr+1+values_per_line*(J-1))
       IF( idate_recrd == 0 ) THEN
          idate_recrd = idate_recrd_prev + 12
       END IF
       idate_recrd_tmp = idate_recrd + 1000000
       WRITE(cdate_recrd_tmp,"(I7)") idate_recrd_tmp
       cdate_recrd = cdate_recrd_tmp(2:7)
       WRITE(UnitNum_Flux,"(A6,4X,12I5,2X,I8)") cdate_recrd, &
         (ifluxs(month_ptr+K+values_per_line*(J-1)),K=1,values_per_line), seq_no_flux
       idate_recrd_prev = idate_recrd
    END DO
  END DO

  ! write FLUXAP section
  DO I = 1, nmonths
    month_ptr = (I-1) * days_for_month
    !write(*,*) 'month_ptr ',month_ptr
    mm_est = (mm_beg - 1) + I
    !write(*,*) 'mm_est ',mm_est
    IF( mm_last == mm_est) THEN
       CYCLE
    END IF
 
    idate_recrd_prev = idate(month_ptr+1)
    DO J = 1,number_lines
       seq_no_fluxap = seq_no_fluxap + 1
       idate_recrd = idate(month_ptr+1+values_per_line*(J-1))
       IF( idate_recrd == 0 ) THEN
          idate_recrd = idate_recrd_prev + 12
       END IF
       idate_recrd_tmp = idate_recrd + 1000000
       WRITE(cdate_recrd_tmp,"(I7)") idate_recrd_tmp
       cdate_recrd = cdate_recrd_tmp(2:7)
       WRITE(UnitNum_FluxAP,"(A6,4X,12I5,2X,I8)") cdate_recrd, &
         (ifluxap(month_ptr+K+values_per_line*(J-1)),K=1,values_per_line), seq_no_fluxap
       idate_recrd_prev = idate_recrd
    END DO
  END DO

  ! write FLUXKP8 section
  DO I = 1, nmonths
    month_ptr = (I-1) * days_for_month
    mm_est = (mm_beg - 1) + I
    !write(*,*) 'mm_est ',mm_est
    IF( mm_last == mm_est) THEN
       CYCLE
    END IF
    DO J = 1, days_for_month
      IF(idate(month_ptr+J) .ne. 0 ) THEN
         seq_no_fluxkp8 = seq_no_fluxkp8 + 1
         idate_recrd_tmp = idate(month_ptr+J) + 1000000
         WRITE(cdate_recrd_tmp,"(I7)") idate_recrd_tmp
         cdate_recrd = cdate_recrd_tmp(2:7)
         WRITE(UnitNum_FluxKP8,"(A6,1X,8(I4,1X),25X,I8)") cdate_recrd, &
              (iflxkp(K,month_ptr+J),K=1,8), seq_no_fluxkp8
      END IF
    END DO
  END DO

    
  END SUBROUTINE Solar_Flux_Pred


!****************************************************************************
  SUBROUTINE Transfer_Master_Section( UnitNum_In, UnitNum_Out, Num_Records, &
                                      FLX, FLXAP, FLXKP, DATE_ARRAY )

  INTEGER(KIND=i4b), INTENT(IN) :: UnitNum_In, UnitNum_Out, Num_Records
  INTEGER(Kind=i4b), INTENT(INOUT), OPTIONAL, DIMENSION(:) :: FLX
  INTEGER(Kind=i4b), INTENT(INOUT), OPTIONAL, DIMENSION(:) :: FLXAP
  INTEGER(Kind=i4b), INTENT(INOUT), OPTIONAL, DIMENSION(:,:) :: FLXKP
  INTEGER(Kind=i4b), INTENT(INOUT), OPTIONAL, DIMENSION(:) :: DATE_ARRAY

  CHARACTER(80) :: Aline

  INTEGER(KIND=i4b) :: I, J, K, IL, nstore_lines, ICNT, idate
  INTEGER(KIND=i4b) :: istart = 1
  INTEGER(KIND=i4b), DIMENSION(8) :: flux_kp

  write(*,*) "in transfer num_records = ", Num_Records
  IF( UnitNum_Out == -1 ) THEN
      ICNT = 0
      DO I = 1, Num_Records
         READ( UnitNum_In, "(A)", IOSTAT = InputStatus ) Aline
         IF (InputStatus /= 0) THEN
           WRITE( Error_UnitNum, * ) "ERROR *** Input error transferring master section with number &
               &records = ", Num_Records
           STOP 16
         END IF
         ! store information based on flag for the last 4 months
         IF( PRESENT( FLX ) ) THEN
             nstore_lines = Num_Records - (4*nlines_per_month) + 1
             IF( I >= nstore_lines ) THEN
                READ( Aline,"(I6,4X,12I5)") DATE_ARRAY(1+ICNT*12), (FLX(J),J=1+ICNT*12,12+ICNT*12)
                 !write(*,*) DATE_ARRAY(1+ICNT*12), (FLX(J),J=1+ICNT*12,12+ICNT*12)
                DO K = 2,12
                  DATE_ARRAY(K+ICNT*12) = DATE_ARRAY(1+ICNT*12) + K - 1
                END DO
                ICNT = ICNT + 1
             END IF
         ELSE IF( PRESENT( FLXAP ) ) THEN
             nstore_lines = Num_Records - (4*nlines_per_month) + 1
             IF( I >= nstore_lines ) THEN
                READ( Aline,"(I6,4X,12I5)") idate, (FLXAP(J),J=1+ICNT*12,12+ICNT*12)
                !write(*,*) idate, (FLX(J),J=1+ICNT*12,12+ICNT*12)
                !write(*,*) idate, DATE_ARRAY(1+ICNT*12)
                IF( idate .NE. DATE_ARRAY(1+ICNT*12) ) THEN
                    WRITE(Error_UnitNum, *) " ERROR *** Problem reading FLUXAP values. "
                    WRITE(Error_UnitNum, *) "  Input Date does not match dates stored from FLUX values."
                    WRITE(Error_UnitNum, *) "  Check program logic. "
                    STOP 16
                END IF
                ICNT = ICNT + 1
             END IF
         ! kp values stored daily
         ELSE IF( PRESENT( FLXKP ) ) THEN
             ICNT = ICNT + 1
             READ( Aline, "(I6,1X,(8(I4,1X)))" ) idate, (flux_kp(J),J=1,8)
             DO IL = istart, tot_days
               IF( idate == DATE_ARRAY(IL) ) THEN
                   FLXKP(:,IL) = flux_kp
                   istart = IL + 1
                   EXIT
               END IF
             END DO
         ELSE
             WRITE(Error_UnitNum) " ERROR *** Incorrect number of arguments passed in to "
             WRITE(Error_UnitNum) "   Subroutine Transfer_Master_Section. "
             STOP 16
         END IF
           
         
      END DO
  ELSE
      DO I = 1, Num_Records
         READ( UnitNum_In, "(A)", IOSTAT = InputStatus ) Aline
         IF (InputStatus /= 0) THEN
           WRITE( Error_UnitNum, * ) "ERROR *** Input error transferring master section with number &
               &records = ", Num_Records
           STOP 16
         END IF
         WRITE(UnitNum_Out,"(A)") Aline
      END DO
  END IF
 
  END SUBROUTINE Transfer_Master_Section



!****************************************************************************
  SUBROUTINE Update_A1UTC( UnitNum, ymd_bih, hms_bih, seq_no )

  INTEGER(KIND=i4b), INTENT(IN) :: UnitNum, ymd_bih, hms_bih
  INTEGER(KIND=i4b), INTENT(INOUT) :: seq_no

  CHARACTER(80) :: Aline
  CHARACTER(20) :: File_Date
  CHARACTER(6) :: aymd
  CHARACTER(5) :: month_name
  CHARACTER(4) :: hhmm
  CHARACTER(2) :: ss

  INTEGER(KIND=i4b) :: ccyy_bih, ccyy_a1utc, mm, I, J, ymd_a1utc
  INTEGER(KIND=i4b) :: taiutc_secs, day, mjd_a1utc, ndays_diff, ymd 
  INTEGER(KIND=i4b) :: seq_no_orig

  REAL(KIND=r8b) :: a1utc_secs, days

  seq_no_orig = seq_no

  ! Open utc-tai file
  OPEN(UNIT = A1UTC_UnitNum, FILE = utctai_file, FORM = "FORMATTED",&
      STATUS = "OLD", IOSTAT = OpenStatus )
  IF (OpenStatus > 0) THEN
    WRITE( Error_UnitNum,* ) "ERROR *** Cannot open input file ", utctai_file
    STOP 16
  END IF

  ccyy_bih = ymd_bih/10000

  ! read utc-tai history file checking for any updates
  ! read header first
  DO 
     READ( A1UTC_UnitNum, "(A)", IOSTAT = InputStatus ) Aline
     IF( InputStatus < 0 ) THEN
       EXIT
     END IF

     IF(INDEX(Aline,"UNTIL") > 0) THEN
        File_Date = Aline(41:62)
        WRITE(Report_UnitNum,*) "   File Date: ", File_Date
        WRITE(Report_UnitNum, *) 
     END IF

     IF( ALine(2:19) == "Limits of validity" ) THEN
        EXIT
     END IF

  END DO
   
  ! read blank line
  READ( A1UTC_UnitNum, "(A)", IOSTAT = InputStatus ) Aline

  DO 
     READ( A1UTC_UnitNum, "(A)", IOSTAT = InputStatus ) Aline
     IF( InputStatus < 0 ) THEN
       EXIT
     END IF

     ! check for last line
     IF( Aline(2:5) == "----" ) THEN
        EXIT
     END IF

     READ(Aline,"(1X,I4)") ccyy_a1utc
     
     ! if year equals or greater need to check for updates
     IF( ccyy_a1utc >= ccyy_bih ) THEN
        
        READ(Aline,"(7X,A5,I2)") month_name, day
        mm = 0
        DO I = 1, 12
          IF( month_name == months_5(I) .OR. month_name == months_5_1(I) ) THEN
             mm = I
             EXIT
          END IF
        END DO

        IF( mm == 0 ) THEN
           WRITE(ERROR_UnitNum,*) "ERROR: Month name, ",month_name, "from ", utctai_file
           WRITE(ERROR_UnitNum,*) "        does not match program array - months."
           STOP 16
        END IF 

        ymd_a1utc = ccyy_a1utc * 10000 + mm * 100 + day
        !write(*,*) "ymd_a1utc ", ymd_a1utc

        ! get number of seconds
        IF( ymd_a1utc > ymd_bih ) THEN
           READ(Aline, "(33X,I3)") taiutc_secs
           !write(*,*) "taiutc_secs ",taiutc_secs
           
           ! compute a1-utc
           a1utc_secs = A1TAI - (-taiutc_secs)

           ! days from Jan 1, 1966
           CALL YMD_MJD( ymd_a1utc, mjd_a1utc)
           ndays_diff = mjd_a1utc - mjd_010166
           days = ndays_diff
           
           ! update seq no
           seq_no = seq_no + 1

           ymd = mod(ymd_a1utc,1000000)
           hhmm = '0000'
           ss = '00'
           WRITE(aymd,"(I6)") ymd
           DO J = 1,6
             IF( aymd(J:J) == " ") aymd(J:J) = "0"
           END DO
           WRITE(UnitNum,"(A6,A4,A2,2X,F16.11,D15.7,D25.16,2X,I8)")  &
             aymd, hhmm, ss, a1utc_secs, zero, days, seq_no

           ! add update to report
           WRITE(Report_UnitNum, "(A)") Aline
           WRITE(Report_UnitNum, "(A)")  "   -------------------------------------------------------"
           WRITE(Report_UnitNum, "('A leap second of ',I3,'s in UTC was introduced on',A6,A4,A2)") &
                    taiutc_secs, aymd, hhmm, ss
           WRITE(Report_UnitNum, "(A/)") "   -------------------------------------------------------"

        END IF

     END IF

  END DO

  ! check to see if any updates were found
  IF( seq_no_orig == seq_no ) THEN
     WRITE(Report_UnitNum, "(A)")  "    --------------------------------------------------"
     WRITE(Report_UnitNum, "(A)")  "    NO positive leap second in UTC has been introduced"
     WRITE(Report_UnitNum, "(A/)") "    --------------------------------------------------"
  END IF


  END SUBROUTINE Update_A1UTC



!****************************************************************************
  SUBROUTINE Update_Header( section, Num_Records )

  CHARACTER(*), INTENT(IN) :: section

  INTEGER(KIND=i4b), INTENT(IN) :: Num_Records

  INTEGER(KIND=i4b), PARAMETER :: kp8_per_line = 8
  INTEGER(KIND=i4b), PARAMETER :: iono_per_line = 3
  INTEGER(KIND=i4b) :: total_kp8, total_iono


  IF( section == "POLE   " ) THEN
     ! set to indicate pole values are in units of micro_arcsecs
     ! the 1st scale flag is for x,y
     ! the 2nd scale flag is for deps and depsilon
     WRITE(NEWBIH_UnitNum,"(A6,4X,3I5)") section, Num_Records, &
          iscale_flag, iscale_flag
  ELSE IF( section == "FLUXKP8" ) THEN
     ! total number of kp values
     total_kp8 = Num_Records * kp8_per_line
     WRITE(NEWBIH_UnitNum,"(A7,2X,2I6)") section, Num_Records, total_kp8
  ELSE IF( section == "IONIND " ) THEN
     ! total number of iono values
     total_iono = Num_Records * iono_per_line
     WRITE(NEWBIH_UnitNum,"(A6,4X,2I5)") section, Num_Records, total_iono
  ELSE
     WRITE(NEWBIH_UnitNum,"(A6,4X,I5)") section, Num_Records
  END IF

  END SUBROUTINE Update_Header


  

!****************************************************************************
  SUBROUTINE Update_Iono( UnitNum_In, UnitNum_Out, record_num, seq_no )

  INTEGER(KIND=i4b), INTENT(IN) :: UnitNum_In, UnitNum_Out
  INTEGER(KIND=i4b), INTENT(IN) :: record_num
  INTEGER(KIND=i4b), INTENT(INOUT) :: seq_no 

  INTEGER(KIND=i4b), PARAMETER :: imax = 962                ! nyears * 12 + 2
  INTEGER(KIND=i4b), DIMENSION(12), PARAMETER :: month = &
                             (/ 31,28,31,30,31,30,31,31,30,31,30,31 /)

  CHARACTER(8) :: cdate
  CHARACTER(7) :: cdate_tmp
  CHARACTER(6) :: cyymmdd

  INTEGER(KIND=i4b) :: iddup, immup, iyyup 
  INTEGER(KIND=i4b) :: imst, iyst, imend, iyend
  INTEGER(KIND=i4b) :: date, date_start, date_end
  INTEGER(KIND=i4b) :: invals, jj, i, iyy, imm, idd
  INTEGER(KIND=i4b) :: numndx, iyywrt, iyymmdd, inum
  INTEGER(KIND=i4b) :: irion, irz, iyymmdd_tmp
  INTEGER(KIND=i4b), DIMENSION(imax) :: iyymmdd_old, irion_old, irz_old

  REAL(KIND=r8b), DIMENSION(imax) :: rion, rz
  REAL(KIND=r8b) :: rrr, zi
  
  seq_no = 0

  REWIND(UnitNum_In)
  DO I = 1, record_num
     READ(UnitNum_In,"(I6,I5,I5)") iyymmdd_old(I), irion_old(I), irz_old(I)
  END DO   

  OPEN(UNIT = IONO_UnitNum, FILE = iono_file, FORM = "FORMATTED",&
      STATUS = "OLD", IOSTAT = OpenStatus )
  IF (OpenStatus > 0) THEN
    WRITE( Error_UnitNum,* ) "ERROR *** Cannot open input file ", iono_file
    STOP 16
  END IF

  READ(IONO_UnitNum, *) iddup, immup, iyyup
  date = iyyup*10000+immup*100+iddup
  
  WRITE(Report_UnitNum,*) "   File Date: ", date
  READ(IONO_UnitNum, *)
 
  READ(IONO_UnitNum, *) imst,iyst,imend,iyend
  date_start = iyst*100+imst
  date_end = iyend*100+imend
  READ(IONO_UnitNum, *)
  WRITE(Report_UnitNum,*)
  WRITE(Report_UnitNum,*) "   ------------------------------------"
  WRITE(Report_UnitNum,*) "   Updated from ",months_3(imst),iyst, "to ",months_3(imend),iyend
  WRITE(Report_UnitNum,*) "   ------------------------------------"
  WRITE(Report_UnitNum,*)

! the first line contains 1 value and the last line contains 1 value
! the remaining lines contains 12 values; this is due to the fact that
! the first year uses a 13-month running mean
!    invals= 12-imst+1+(iyend-iyst-1)*12 +imend + 2
!             1st year \ full years       \last y\ before & after

  invals=(iyend-iyst)*12 +imend-imst+3

  READ(IONO_UnitNum,*) (rion(i),i=1,invals)
  READ(IONO_UnitNum,*) (rz(i),i=1,invals)

  DO jj=1,invals 
    IF( rion(jj).le.-90.0_r8b ) THEN
        rrr=rz(jj)
        zi=-12.349154_r8b+(1.4683266_r8b-2.67690893e-03_r8b*rrr)*rrr
        IF(zi.gt.274.0) zi=274.0_r8b
        rion(jj)=zi
    END IF
  END DO

  CLOSE(IONO_UnitNum)
 
  imm = imst - 1
  iyy = iyst
  IF( imm .eq. 0 ) THEN
    imm = 12
    iyy = iyst - 1
  END IF 
  idd=month(imm)
  IF(mod(mod(iyy,100),4).eq.0.and.imm.eq.2)idd=29
  iyymmdd=iyy*10000+imm*100+idd

  inum = 1
  IF( mod(iyymmdd,1000000) > iyymmdd_old(1) ) THEN
     DO I = inum, record_num
        ! check for date match then data match
        seq_no = seq_no+1
        WRITE(cdate,"(I8)") iyymmdd_old(I)
        cyymmdd = cdate(3:8)
        WRITE(UnitNum_Out,"(A6,I5,I5,59X,I5)")cyymmdd, irion_old(I), irz_old(I), seq_no
        inum = I+1
        IF( mod(iyymmdd,100) .eq. iyymmdd_old(I) ) THEN
           EXIT
        END IF
     END DO 
  END IF
      
  !write(*,*) 'inum iyymmdd_old(1) ',inum,iyymmdd_old(1)
  DO numndx=1,invals
     idd=month(imm)
     IF(mod(mod(iyy,100),4).eq.0.and.imm.eq.2)idd=29
     iyymmdd=iyy*10000+imm*100+idd
     irion=rion(numndx)*10
     irz=rz(numndx)*10
     IF(irz.lt.0) THEN
        WRITE(Error_UnitNum,*) "ERROR:  The value of ionospheric indices for ",iyymmdd, " equals zero."
        WRITE(Error_UnitNum,*) "  This is not a valid number. Check the file :",iono_file
        STOP 16
     END IF
  
     DO I = inum, record_num
        ! check for date match then data match
        IF( mod(iyymmdd,1000000) .eq. iyymmdd_old(I) ) THEN
            IF( irion /= irion_old(I) .OR. irz /= irz_old(I) ) THEN
               WRITE(cdate,"(I8)") iyymmdd
               cyymmdd = cdate(3:8)
               WRITE(Report_UnitNum,*) '   Ionospheric indices have changed for ',cyymmdd
               WRITE(Report_UnitNum,*) '   Indices were         ',irion_old(I),' and ',irz_old(I)
               WRITE(Report_UnitNum,*) '   Indices changed to   ',irion,' and ',irz
            END IF
            inum = I+1
            EXIT
        END IF
     END DO 

     seq_no = seq_no+1
     WRITE(cdate,"(I8)") iyymmdd
     cyymmdd = cdate(3:8)
     WRITE(UnitNum_Out,"(A6,I5,I5,59X,I5)")cyymmdd, irion, irz, seq_no
     imm=imm+1
     IF( imm > 12 ) THEN
        imm = 1
        iyy=iyy+1
     END IF
  END DO

  !write(*,*) 'inum ',inum
  IF( mod(iyymmdd,1000000) < iyymmdd_old(inum) ) THEN
     DO I = inum, record_num
        ! check for date match then data match
        seq_no = seq_no+1
        ! add value to get zero fill for date
        iyymmdd_tmp = iyymmdd_old(I) + 1000000
        WRITE(cdate_tmp,"(I7)") iyymmdd_tmp
        cyymmdd = cdate_tmp(2:7)
        WRITE(UnitNum_Out,"(A6,I5,I5,59X,I5)")cyymmdd, irion_old(I), irz_old(I), seq_no
        inum = I
        IF( iyymmdd .eq. iyymmdd_old(I) ) THEN
           EXIT
        END IF
     END DO 
   END IF


  END SUBROUTINE Update_Iono



!****************************************************************************
  SUBROUTINE Update_Pole( UnitNum, UnitNum2, seq_no )

! COMMENTS:
!          UT1 Zonal Tide effects for periods less than 35 days are removed
!          from the UT1 data acquired from eopc04 files.
!          The IERS algorithm is used (Page 72 of IERS Technical Note 21).
!          The constants, coefficients and algorithms must match what GEODYN
!          is doing to add the corrections back in.
!          DO NOT change this computation unless you also change it in GEODYN
!          IIE routine UTTIDE
!
!          dPsi and dEpsilon have been replace with dX and dY; computations 
!          must be included to convert dX, dY to dPsi, dEpsilon


  INTEGER(KIND=i4b), INTENT(IN) :: UnitNum, UnitNum2
  INTEGER(KIND=i4b), INTENT(INOUT) :: seq_no

  CHARACTER(90) :: Aline, Date_Line
  CHARACTER(6) :: aymd
  CHARACTER(2) :: hh

  INTEGER(KIND=i4b) :: ccyy, mjd_eop, ymd_bih, year, mjd_bih, cymd_eop
  INTEGER(KIND=i4b) :: Num_Records, icurr = 0
  INTEGER(KIND=i4b) :: I, J, ymd, ymdh
  INTEGER(KIND=i4b) :: ixout, iyout, idpsio, idepso
  INTEGER(KIND=i4b) :: iopt

  REAL(KIND=r8b) :: xin, yin, ut1utc, dlod, dpsi, deps
  REAL(KIND=r8b) :: xout, yout, dpsio, depso, xmjd
  REAL(KIND=r8b) :: a1utc, a1ut1, dut1
  REAL(KIND=r8b) :: dmjd, dX, dY

  LOGICAL :: LFIRST

  ! Open pole file
  OPEN(UNIT = POLE_Curr_UnitNum, FILE = pole_curr_file, FORM = "FORMATTED",&
      STATUS = "OLD", IOSTAT = OpenStatus )
  IF (OpenStatus > 0) THEN
    WRITE( Error_UnitNum,* ) "ERROR *** Cannot open input file ", pole_curr_file
    STOP 16
  END IF
  ! read file header
  Pole_Hdr: DO I = 1,14
     READ( POLE_Curr_UnitNum, "(A)", IOSTAT = InputStatus ) Aline
     IF( InputStatus < 0 ) THEN
       EXIT Pole_Hdr
     END IF
  END DO Pole_Hdr

  ! read data records
  LFIRST = .FALSE.
  Pole_Data: DO
     READ( POLE_Curr_UnitNum, "(A)", IOSTAT = InputStatus ) Aline
     IF( InputStatus < 0 ) THEN
       EXIT Pole_Data
     END IF

     !READ(ALine,"(17X,I5)") mjd_eop
     READ(ALine,"(12X,I7)") mjd_eop
     IF( mjd_eop == 0 ) THEN 
       CYCLE
     END IF

     ! transfer pole section not being updated
     IF( .NOT. LFIRST ) THEN
         REWIND( UnitNum )
         READ(UnitNum, "(A)") Date_Line
         READ(Date_Line,"(I6)") ymd_bih
         year = ymd_bih/10000
         if(year < 50) then
           ymd_bih = 20000000 + ymd_bih
         else
           ymd_bih = 19000000 + ymd_bih
         end if
         CALL YMD_MJD( ymd_bih, mjd_bih)
         !write(*,*) 'mjd_bih ', mjd_bih
         Num_Records = ( mjd_eop -  mjd_bih )
         !write(*,*) "num_records ",Num_Records
         LFIRST = .TRUE.
         REWIND( UnitNum )
         !CALL Transfer_Master_Section( UnitNum, UnitNum2, Num_Records )
         ! ensure file is reformatted to new format
         DO I = 1, Num_Records
            READ(UnitNum, *) ymdh,ixout,iyout,a1ut1,idepso,idpsio,seq_no
            WRITE(UnitNum2, "(I8,2(1X,I7),F15.10,4X,2I7,15X,I8)")  &
                              ymdh,ixout,iyout,a1ut1,idepso,idpsio,seq_no
         END DO

         CALL MJD_YMD(mjd_eop, cymd_eop)
         WRITE(Report_UnitNum, "('    -----------------------------------')")
         WRITE(Report_UnitNum, "('    Updated from ',I8)", ADVANCE="NO") cymd_eop
         
     END IF

     ! update remaining file
     !READ(Aline,"(11X,I5,2F9.5,F10.6,2X,F10.6,2X,2F9.5)") &
     !READ(Aline,"(17X,I5,2F9.6,F10.7,2X,F10.7,2X,2F9.6)") &
     READ(Aline,"(12X,I7,2(F11.6),2(F12.7),2(F11.6))") &
        mjd_eop, xin, yin, ut1utc, dlod, dX, dY

     !write(*,*)  mjd_eop, xin, yin, ut1utc, dlod, dX, dY
     icurr=icurr+1

     ! change scale to get more precision (was ScaleD4) for xout, yout
     xout = xin*ScaleD6
     yout = yin*ScaleD6

     ! set conversion flag to go from dX,dY to dpsi,deps
     iopt = 2
     dmjd = REAL(mjd_eop,r8b)
     call dXdY_dpsideps(dmjd,dX,dY,dpsi,deps,iopt)
     dpsio = dpsi*ScaleD6
     depso = deps*ScaleD6

     !write(*,*) mjd_eop, dX, dY, dpsi, deps

     !write(*,*) 'dpsio depso ',dpsio, depso
     ixout = xout
     iyout = yout
     idpsio = dpsio
     idepso = depso

     !write(*,*) 'idpsio idepso ',idpsio, idepso
     CALL Calculate_a1ut1( mjd_eop, ut1utc, a1ut1 )
!
     CALL MJD_YMD(mjd_eop, cymd_eop)

     ymd = mod(cymd_eop,1000000)
  
     seq_no = Num_Records+icurr
     hh = '00'
     WRITE(aymd,"(I6)") ymd
     DO J = 1,6
        IF( aymd(J:J) == " ") aymd(J:J) = "0"
     END DO

     !WRITE(UnitNum2, "(A6,A2,2X,2I5,D15.6,4X,2I6,21X,I8)")  &  old format
     WRITE(UnitNum2, "(A6,A2,2(1X,I7),F15.10,4X,2I7,15X,I8)")  &
                           aymd,hh,ixout,iyout,a1ut1,idepso,idpsio,seq_no


  END DO Pole_Data
  
  WRITE(Report_UnitNum, "(' to ',I8)") cymd_eop
  WRITE(Report_UnitNum, "( '    -----------------------------------')")

  END SUBROUTINE Update_Pole




!****************************************************************************
  SUBROUTINE Update_Solar_Flux( FLUX_UnitNum, UnitNum_Flux,                 &
                                UnitNum_FluxAP,  UnitNum_FluxKP8,           &
                                seq_no_flux, seq_no_fluxap, seq_no_fluxkp8, &
                                last_date_flux )

  INTEGER, INTENT(IN) :: FLUX_UnitNum, UnitNum_Flux
  INTEGER, INTENT(IN) :: UnitNum_FluxAP, UnitNum_FluxKP8
  INTEGER, INTENT(INOUT) :: seq_no_flux, seq_no_fluxap, seq_no_fluxkp8
  INTEGER, INTENT(INOUT) :: last_date_flux

  INTEGER, PARAMETER :: tot_days_year = 432     ! 12 months x 36 days
  INTEGER, PARAMETER :: number_lines = 3        ! 3 lines of 12 values
  INTEGER, PARAMETER :: days_for_month = 36     ! 36 days for storage for each month
  INTEGER, PARAMETER :: values_per_line = 12    ! 12 values per line
  INTEGER, PARAMETER :: kp8_incrmt = 33         ! increment of kp8 values

  CHARACTER(10) :: date
  CHARACTER(15) :: filnam
  CHARACTER(80) :: Aline
  CHARACTER(7) :: cdate_recrd_tmp
  CHARACTER(6) :: cdate_recrd
   
  REAL(KIND=r8b) :: fluxs_recrd

  INTEGER(Kind=i4b), DIMENSION(tot_days_year) :: ifluxap, ifluxs
  INTEGER(Kind=i4b), DIMENSION(8):: inckpv = 0
  INTEGER(Kind=i4b), DIMENSION(8,tot_days_year):: iflxkp
  INTEGER(Kind=i4b), DIMENSION(8):: iflxkp_recrd = 0
  INTEGER(Kind=i4b), DIMENSION(tot_days_year):: kpsum, idate
  INTEGER(Kind=i4b) :: kpsum_recrd, idate_recrd, idate_recrd_tmp
  INTEGER(Kind=i4b) :: inckps, iapsum, j, I, k
  INTEGER(Kind=i4b) :: immdd, month_prev, month, iday
  INTEGER(Kind=i4b) :: mm_beg, mm_end, iyy
  INTEGER(Kind=i4b) :: array_ptr, month_ptr
  INTEGER(Kind=i4b) :: date_begin
 
  LOGICAL :: LFIRST

  ! initialize variables
  LFIRST = .FALSE.
  ifluxap = 0
  ifluxs = 0
  iflxkp = 0
  kpsum = 0
  idate = 0

  DO
    READ( FLUX_UnitNum, "(A)", IOSTAT = InputStatus ) Aline
    IF( InputStatus < 0 ) THEN
      EXIT
    END IF
    IF (InputStatus /= 0) THEN
      WRITE( Error_UnitNum, * ) "ERROR *** Input file error reading on unit ", &
          FLUX_UnitNum
      STOP 16
    END IF

    READ(ALINE,"(i6,6x,8(i1,i1),i2,i1,24x,i3,7x,f5.1)" )         &
          idate_recrd,(iflxkp_recrd(j),                          &
          inckpv(j),j=1,8),kpsum_recrd,inckps,iapsum,fluxs_recrd

!   WRITE(*,"(i6,6x,8(i1,i1),3x,i2,i1,4x,i3,7x,f5.1)" )         &
!             idate_recrd,(iflxkp_recrd(j),                          &
!             inckpv(j),j=1,8),kpsum_recrd,inckps,iapsum,fluxs_recrd

    iyy = idate_recrd/10000
    immdd = mod(idate_recrd,10000)
    month = immdd/100
    iday = immdd - month*100
    
    array_ptr = ((month-1)*days_for_month) + iday
    IF( .NOT. LFIRST ) THEN
       mm_beg = month
       date_begin = idate_recrd
       IF( iyy < 50 ) THEN
          date_begin = date_begin + 20000000
       ELSE
          date_begin = date_begin + 20000000
       END IF
       LFIRST = .TRUE.
    ELSE
       mm_end = month
    END IF
   

    idate(array_ptr) = idate_recrd
 
    DO i=1,8
       iflxkp(i,array_ptr) = iflxkp_recrd(i)
       if(inckpv(i).eq.7) then
         iflxkp(i,array_ptr)=((iflxkp(i,array_ptr) + 1) * 100) - kp8_incrmt
       else
         if(inckpv(i).eq.3) then 
           iflxkp(i,array_ptr)=(iflxkp(i,array_ptr) * 100) + kp8_incrmt
         else 
           iflxkp(i,array_ptr)=(iflxkp(i,array_ptr) * 100)
         endif
       endif
    END DO
    ifluxap(array_ptr)=iapsum
    ifluxs(array_ptr)=fluxs_recrd*10.0
    
  END DO

  ! store end time
  last_date_flux = idate_recrd
  IF( iyy < 50 ) THEN
     last_date_flux = last_date_flux + 20000000
  ELSE
     last_date_flux = last_date_flux + 19000000
  END IF
    
  DO I = mm_beg, mm_end
    month_ptr = (I-1) * days_for_month
    DO J = 1,number_lines
       seq_no_flux = seq_no_flux + 1
       idate_recrd_tmp = idate(month_ptr+1+values_per_line*(J-1)) + 1000000
       WRITE(cdate_recrd_tmp,"(I7)") idate_recrd_tmp
       cdate_recrd = cdate_recrd_tmp(2:7)
       WRITE(UnitNum_Flux,"(A6,4X,12I5,2X,I8)") cdate_recrd,  &
         (ifluxs(month_ptr+K+values_per_line*(J-1)),K=1,values_per_line), seq_no_flux
    END DO
  END DO

  DO I = mm_beg, mm_end
    month_ptr = (I-1) * days_for_month
    DO J = 1,number_lines
      seq_no_fluxap = seq_no_fluxap + 1
       idate_recrd_tmp = idate(month_ptr+1+values_per_line*(J-1)) + 1000000
       WRITE(cdate_recrd_tmp,"(I7)") idate_recrd_tmp
       cdate_recrd = cdate_recrd_tmp(2:7)
      WRITE(UnitNum_FluxAP,"(A6,4X,12I5,2X,I8)") cdate_recrd,  &
         (ifluxap(month_ptr+K+values_per_line*(J-1)),K=1,values_per_line), seq_no_fluxap
    END DO
  END DO

  DO I = mm_beg, mm_end
    month_ptr = (I-1) * days_for_month
    DO J = 1, days_for_month
      IF(idate(month_ptr+J) .ne. 0 ) THEN
         seq_no_fluxkp8 = seq_no_fluxkp8 + 1
       idate_recrd_tmp = idate(month_ptr+J) + 1000000
       WRITE(cdate_recrd_tmp,"(I7)") idate_recrd_tmp
       cdate_recrd = cdate_recrd_tmp(2:7)
         WRITE(UnitNum_FluxKP8,"(A6,1X,8(I4,1X),25X,I8)") cdate_recrd, &
            (iflxkp(K,month_ptr+J),K=1,8), seq_no_fluxkp8
      END IF
    END DO

  END DO

  WRITE(Report_UnitNum,*)
  WRITE(Report_UnitNum,*) "   ----------------------------------"
  WRITE(Report_UnitNum,*) "   Updated from",date_begin, "to",last_date_flux
  WRITE(Report_UnitNum,*) "   ----------------------------------"
  WRITE(Report_UnitNum,*)
    
  END SUBROUTINE Update_Solar_Flux



!****************************************************************************
  FUNCTION UTTIDE(XMJD)
!
! FUNCTION:  CALCULATE THE ZONAL TIDE EFFECT ON UT TIME FOR
!            TIDAL CONSTITUENTS WHICH HAVE A PERIOD OF 35 DAYS
!            OR LESS.IT IS BELIEVED THAT THE EFFECT OF THESE 41
!            CONSTITUENTS IS NOT PRESENT IN THE BIH TABLES AND THAT
!            ANY FIVE DAY TABLE WOULD HAVE TROUBLE REPRESENTING
!            THESE SHORT PERIOD EFFECTS.REFERENCE IS THE BIH CIRCULAR
!            D NOTICE TO USERS,MARCH 5,1982.
!
!
!
! COMMENTS:
!             Modified from the GEODYN IIE routine UTTIDE
!
   REAL(KIND=r8b) :: UTTIDE
   REAL(KIND=r8b), INTENT(IN) :: xmjd

   REAL(KIND=r8b), PARAMETER :: DEGRAD = 1.7453292519943E-2
   REAL(KIND=r8b), PARAMETER :: ARGL0 = 134.9629D0, ARGLP0 = 357.5254D0
   REAL(KIND=r8b), PARAMETER :: ARGF0 = 93.2728D0
   REAL(KIND=r8b), PARAMETER :: ARGD0 = 297.8503D0, ARGW0 = 125.0433D0
   REAL(KIND=r8b), PARAMETER :: ARGLS = 13.06499295D0, ARGLPS = 0.985600258D0
   REAL(KIND=r8b), PARAMETER :: ARGFS = 13.22935027D0, ARGDS = 12.19074911D0
   REAL(KIND=r8b), PARAMETER :: ARGWS = 0.05295381D0
   REAL(KIND=r8b), PARAMETER :: ak = 0.940D0
   REAL(KIND=r8b), DIMENSION(41) ::  XARG, SARG
   REAL(KIND=r8b) :: XTIME, ARGL, ARGLP, ARGF, ARGD, ARGW, DEL2

   INTEGER(KIND=i4b) :: I

! The following are from Yoder, eta al., 1981
!   make sure multiply by 0.94(k/c) and 1.0D-7(units)
!
   REAL(KIND=r8b), DIMENSION(41), PARAMETER :: UTTDAM =               &
                 (/-0.2500d+02,-0.4300d+02,-0.1050d+03,-0.5400d+02,   &
                   -0.1310d+03,                                       &
                -0.41000d+02,-0.43700d+03,-0.10560d+04,-0.19000d+02,  &
                -0.87000d+02,-0.21000d+03,-0.81000d+02, 0.23000d+02,  &
                 0.27000d+02,-0.31800d+03,-0.34130d+04,-0.82520d+04,  &
                 0.23000d+02,-0.36000d+03, 0.19000d+02,-0.26000d+02,  &
                +0.50000d+02,-0.78100d+03,-0.56000d+02,-0.54000d+02,  &
                 0.53000d+02, 0.10700d+03, 0.42000d+02, 0.50000d+02,  &
                 0.18800d+03, 0.46300d+03, 0.56800d+03,-0.87880d+04,  &
                 0.57900d+03, 0.50000d+02,-0.59000d+02, 0.12500d+03,  &
                -0.19400d+04, 0.14000d+03, 0.19000d+02,-0.91000d+02/)

   REAL(KIND=r8b), DIMENSION(41), PARAMETER :: XL =                   &
            (/1.D0,2.D0,2.D0,0.D0,0.D0,1.D0,1.D0,1.D0,3.D0,-1.D0,     &
             -1.D0,1.D0,2.D0,0.D0,0.D0,0.D0,0.D0,2.D0,2.D0,2.D0,      &
              0.D0,0.D0,0.D0,0.D0,0.D0,1.D0,1.D0,1.D0,-1.D0,-1.D0,    &
             -1.D0,1.D0,1.D0,1.D0,0.D0,1.D0,-1.D0,-1.D0,-1.D0,1.D0,   &
             -1.D0/)

   REAL(KIND=r8b), DIMENSION(41), PARAMETER :: XLP =                  &
             (/0.D0,0.D0,0.D0,0.D0,0.D0,0.D0,0.D0,0.D0,0.D0,0.D0,     &
               0.D0,0.D0,0.D0,1.D0,0.D0,0.D0,0.D0,0.D0,0.D0,0.D0,     &
              -1.D0,0.D0,0.D0,0.D0,-1.D0,0.D0,0.D0,1.D0,0.D0,0.D0,    &
               0.D0,0.D0,0.D0,0.D0,0.D0,-1.D0,0.D0,0.D0,0.D0,0.D0,    &
              -1.D0/)

   REAL(KIND=r8b), DIMENSION(41), PARAMETER :: XF =                   &
            (/2.D0,2.D0,2.D0,2.D0,2.D0,2.D0,2.D0,2.D0,0.D0,2.D0,      &
              2.D0,0.D0,2.D0,2.D0,2.D0,2.D0,2.D0,0.D0,0.D0,0.D0,      &
              2.D0,0.D0,0.D0,0.D0,0.D0,2.D0,2.D0,0.D0,2.D0,2.D0,      &
              2.D0,0.D0,0.D0,0.D0,0.D0,0.D0,0.D0,0.D0,0.D0,-2.D0,     &
              0.D0/)

   REAL(KIND=r8b), DIMENSION(41), PARAMETER :: XD =                   &
            (/2.D0,0.D0,0.D0,2.D0,2.D0,0.D0,0.D0,0.D0,0.D0,2.D0,      &
              2.D0,2.D0,-2.D0,0.D0,0.D0,0.D0,0.D0,0.D0,0.D0,0.D0,     &
              0.D0,2.D0,2.D0,2.D0,2.D0,-2.D0,-2.D0,0.D0,0.D0,0.D0,    &
              0.D0,0.D0,0.D0,0.D0,1.D0,0.D0,2.D0,2.D0,2.D0,2.D0,      &
              2.D0/)

   REAL(KIND=r8b), DIMENSION(41), PARAMETER :: XW =                   &
            (/2.D0,1.D0,2.D0,1.D0,2.D0,0.D0,1.D0,2.D0,0.D0,1.D0,      &
              2.D0,0.D0,2.D0,2.D0,0.D0,1.D0,2.D0,-1.D0,0.D0,1.D0,     &
              2.D0,-1.D0,0.D0,1.D0,0.D0,1.D0,2.D0,0.D0,0.D0,1.D0,     &
              2.D0,-1.D0,0.D0,1.D0,0.D0,0.D0,-1.D0,0.D0,1.D0,-1.D0,   &
              0.D0/)

      XTIME=(XMJD-51544.50D0)

      ARGL=ARGL0+XTIME*ARGLS 
      ARGLP=ARGLP0+XTIME*ARGLPS
      ARGF=ARGF0+XTIME*ARGFS
      ARGD=ARGD0+XTIME*ARGDS
      ARGW=ARGW0-XTIME*ARGWS

      DO I = 1,41
        XARG(I)=(ARGL*XL(I)+ARGLP*XLP(I)+ARGF*XF(I)+ARGD*XD(I)+ARGW*XW(I))*DEGRAD
      END DO
 

      DO I = 1,41
        SARG(I)=DSIN(XARG(I))
      END DO 

      DEL2=ZERO
      DO I = 1,41
        DEL2=DEL2+ak*1.0D-07*UTTDAM(I)*SARG(I)
      END DO

      UTTIDE = DEL2

  END FUNCTION UTTIDE

END PROGRAM bih_master_update

